/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.core.component;

import java.io.IOException;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.render.Renderer;

import org.apache.commons.logging.Log;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.UIComponentFactory;
import org.exoplatform.faces.context.PortletExternalContext;
import org.exoplatform.faces.core.Util;
import org.exoplatform.faces.core.renderer.html.Decorator;
import org.exoplatform.portal.Constants;
import org.exoplatform.portal.PortalInfo;
import org.exoplatform.services.log.LogService;

/**
 * Wed, Dec 22, 2003 @ 23:14 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIExoComponentBase.java,v 1.28 2004/10/27 02:52:15 tuan08 Exp $
 */
public class UIExoComponentBase extends UIComponentBase implements UIExoComponent {
  private String baseURL_ ;
  private String clazz_;
  private Decorator decorator_ ;
  private transient Renderer cacheRenderer_ ;

  public String getClazz() { return clazz_ ; }
  public UIExoComponent setClazz(String clazz) { 
    clazz_ = clazz  ; 
    return this ;
  }

  public String getFamily() { return UIExoComponent.COMPONENT_FAMILY ; }
  
  public ResourceBundle getApplicationResourceBundle(ExternalContext eContext) {
    if (eContext instanceof PortletExternalContext) {
      PortletExternalContext impl = (PortletExternalContext) eContext;
      return impl.getApplicationResourceBundle();
    } 
    return (ResourceBundle) eContext.getRequestMap().get(Constants.APPLICATION_RESOURCE);
  }
  
  public boolean canDecodeInvalidState()  { return true ; }
  
  public UIExoComponent setDecorator(Decorator decorator) { 
    decorator_ = decorator ; 
    return this ;
  }
  
  public void processDecodes(FacesContext context) {
    List children = getChildren() ;
    for(int i = 0 ; i < children.size(); i++) {
      UIComponent child = (UIComponent) children.get(i);
      if (child.isRendered()) {
        child.processDecodes(context) ;
        if (context.getRenderResponse()) 	return ;
      }
    }
    decode(context) ;
  }
  
  public void setRendererType(String type) {
    cacheRenderer_ = null ;
    super.setRendererType(type) ;
  }
  
  protected Renderer getRenderer(FacesContext context) {
    if(cacheRenderer_ == null) {
      cacheRenderer_ = Util.getRenderer(context, getRendererType(), getFamily());
    }
    return cacheRenderer_  ;
  }
  
  public UIExoComponent findComponentById(String id) {
    if(id.equals(getId())) return this ; 
    List children = getChildren();
    for (int i = 0; i < children.size() ; i++) {
      UIExoComponent uiComponent = (UIExoComponent) children.get(i) ;
      UIExoComponent result = uiComponent.findComponentById(id) ;
      if (result != null) return result ;
    }
    return null ;
  }
  
  public UIExoComponent findRenderedComponentById(String id) {
    if(id.equals(getId())) return this ; 
    List children = getChildren();
    for (int i = 0; i < children.size() ; i++) {
      UIExoComponent uiComponent = (UIExoComponent) children.get(i) ;
      if(uiComponent.isRendered()) {
        UIExoComponent result = uiComponent.findComponentById(id) ;
        if (result != null) return result ;
      }
    }
    return null ;
  }
  
  
  public UIComponent getAncestorOfType(Class classType) {
  	UIComponent parent = getParent() ;
  	while(parent != null ) {
  		if( classType.isInstance(parent)) return parent ;
  		parent = parent.getParent() ;
  	}
    return null ;
  }  
  
  public Object getChildComponentOfType(Class classType) {
    List children = getChildren() ;
    for (int i = 0; i < children.size(); i++) {
      Object o = children.get(i)  ;
      if( classType.isInstance(o)) return o ;
    }
    return null ;
  }
  
  public void setRenderedComponent(String id) {
  	List children = getChildren() ;
  	for(int i = 0 ; i < children.size(); i++) {
  		UIComponent child = (UIComponent) children.get(i);
  		if(child.getId().equals(id))  {
  			child.setRendered(true);
  		} else  {
  			child.setRendered(false) ;
  		}
  	}
  }
  
  public void setRenderedComponent(Class type) {
    List children = getChildren() ;
    for(int i = 0 ; i < children.size(); i++) {
      UIComponent child =  (UIComponent)children.get(i);
      if(type.isInstance(child))  child.setRendered(true);
      else   child.setRendered(false) ;
    }
  }
  
  public void setRenderedSibling(Class type) {
    List children = getParent().getChildren() ;
    for(int i = 0 ; i < children.size(); i++) {
      UIComponent child =  (UIComponent)children.get(i);
      if(type.isInstance(child))  child.setRendered(true);
      else   child.setRendered(false) ;
    }
  }
  
  public UIComponent getSibling(Class type) {
    List children = getParent().getChildren() ;
    for(int i = 0 ; i < children.size(); i++) {
      UIComponent child =  (UIComponent)children.get(i);
      if(type.isInstance(child))  return child  ;
    }
    return null ;
  }
  
  public String getBaseURL(FacesContext context) {
    if (baseURL_ == null) {
      ExternalContext econtext = context.getExternalContext() ;
      if(econtext instanceof PortletExternalContext) {
        baseURL_ = econtext.encodeActionURL("") ;
        baseURL_ += Constants.AMPERSAND + UICOMPONENT + "=" + getId() ;
      } else {
        PortalInfo portalInfo = SessionContainer.getInstance().getPortalInfo();
        baseURL_ = portalInfo.getOwnerURI();
        baseURL_ += "?" + UICOMPONENT + "=" + getId() ;
      }
    }
    return baseURL_ ;
  }
  
  public String getBaseURL() {
    if (baseURL_ == null) {
      baseURL_ = getBaseURL(FacesContext.getCurrentInstance());
    }
    return baseURL_ ;
  }

  public void setBaseURL(String baseURL) { 
    baseURL_ = baseURL + Constants.AMPERSAND + UICOMPONENT + "=" + getId()  ;  
  }
  
  public void decorate(FacesContext context) throws IOException {
    if(decorator_ != null) {
      decorator_.decorate(context, this) ;
    } else {
      encodeBegin(context) ;
      encodeChildren(context) ;
      encodeEnd(context) ;
    }
  }
  
  public UIExoComponent addChild(Class cl) throws Exception {
    UIExoComponent uiComponent =  UIComponentFactory.createComponent(cl) ;
    getChildren().add(uiComponent) ;
    uiComponent.registerActionListener(this) ;
    uiComponent.registerComponentObserver(this) ;
    return uiComponent ;
  }
  
  public void registerActionListener(UIExoComponent parent)  {
    
  }
  
  public void registerComponentObserver(UIExoComponent parent)  {
    
  }
  
  static public Log getLog(String name)  {
    PortalContainer manager  = PortalContainer.getInstance();
    LogService service = (LogService) manager.getComponentInstanceOfType(LogService.class) ;
    Log log = service.getLog(name);
    return log ;
  }
}
