/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.  *
 **************************************************************************/
package org.exoplatform.faces.core.renderer.html;

import java.io.IOException;
import java.util.ResourceBundle ;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.event.PhaseId;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import org.exoplatform.faces.FacesConstants;
import org.exoplatform.faces.core.Util;
import org.exoplatform.faces.core.component.UIGrid;
import org.exoplatform.faces.core.component.UIHiddenInput;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.model.FormButton;
import org.exoplatform.faces.core.component.model.Row;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.commons.utils.ExpressionUtil ;

public  class SimpleFormRenderer extends HtmlBasicRenderer {
  
	public void decode(FacesContext context, UIComponent component ) {
		UISimpleForm uiForm = (UISimpleForm) component ; 
		Map paramMap = context.getExternalContext().getRequestParameterMap() ;
		uiForm.setUserAction((String) paramMap.get(FacesConstants.ACTION));
		String action = uiForm.getUserAction() ;
		ExoActionEvent event =new ExoActionEvent(uiForm, action, paramMap);
		 if(Util.getActionPhaseId(action) == PhaseId.APPLY_REQUEST_VALUES) {
		  uiForm.broadcast(event) ;
		} else {
		  uiForm.queueEvent(event) ;
		}
	}
  
  public void encodeChildren( FacesContext context, UIComponent component ) throws IOException {
    ResponseWriter w = context.getResponseWriter() ;
    ResourceBundle res = getApplicationResourceBundle(context.getExternalContext()) ;
    UISimpleForm uiForm = (UISimpleForm) component ; 
    String actionURL = uiForm.getBaseURL(context) ;
    String formName = uiForm.getFormName() ;
    //String formId = uiForm.getFormId() ;
    String method = uiForm.getMethod() ;
    List hiddenInputs = uiForm.getHiddenInputs() ;
    String clazz = uiForm.getClazz() ;
    if(clazz == null) clazz = "" ;
    w.write("<form name='"); w.write(formName); w.write("'") ;
    w.write(" action='"); w.write(actionURL); w.write("'") ;
    if(method != null){ 
      w.write(" method='"); w.write(method); w.write("'") ;
    }
    w.write(" class='UISimpleForm");
    if(clazz != null)  {
      w.write(' ') ; w.write(clazz) ;
    }
    w.write("'") ; 
    w.write(">\n");
    w.write("<input type='hidden' name='"); w.write(ACTION); w.write("'") ;
    w.write(" value=''/>\n") ;
    for (int i = 0; i < hiddenInputs.size(); i++) {
      UIHiddenInput uiInput = (UIHiddenInput) hiddenInputs.get(i) ;
      uiInput.encodeBegin(context) ;
    }
    w.  write("<table>\n") ;
    List rows = uiForm.getRows() ;
    for (int i = 0 ; i < rows.size() ; i++) {
      Row row = (Row) rows.get(i) ;
      row.render(w, res, uiForm) ;
    }
    w.  write("</table>") ;
    w.write(getScript(uiForm));
    w.write("</form>") ;
  }
  
  public void renderFormButton(FormButton button, ResourceBundle res, 
                               ResponseWriter w, UIGrid uiParent)  throws IOException {
      UISimpleForm uiForm = (UISimpleForm) uiParent ;
      String label = ExpressionUtil.getExpressionValue(res, button.getLabel()) ;
      w.write("<a");
      if (button.getClazz() != null) {
        w.write(" class='"); w.write(button.getClazz()); w.write("'");
      } else {
        w.write(" class='"); w.write(getDefaultClass()); w.write("'");
      }
      w.write(" href=\"javascript:submit_"); w.write(uiForm.getFormName()); w.write("('");
      w.write(button.getAction()) ; w.write("');\">");
      w.write(formatText(label));
      w.write("</a>");
  }
  
  protected String formatText(String text) { 
    return text ;   
  }
  
  protected String getDefaultClass() { return "form-link" ; }
  
  protected String getScript(UISimpleForm uiForm) {
    return uiForm.getScript() ;
  }
}