/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.faces.user.component;

import javax.faces.context.ExternalContext;
import javax.portlet.ActionResponse;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.ExoPortalInfo;
import org.exoplatform.services.portletcontainer.pci.Output;
import org.exoplatform.services.security.SecurityService;
/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 1 mai 2004
 */
public class UILogin extends UISimpleForm {
  public static final String LOGIN_ACTION = "login";
  public static final String REGISTER_ACTION = "register";
  public static final String LOGOUT_ACTION = "logout";

  private UIStringInput userNameInput_;
  private UIStringInput passwordInput_;

  private String loginPath_;
  private SecurityService securityService_;

  public UILogin(SecurityService service) throws Exception {
    super("loginForm", "post", null);
    setId("UILogin");
    setRendererType("SimpleFormRenderer");
    setClazz("UILogin");
    securityService_ = service ;
    ExoPortalInfo epinfo = ExoPortalInfo.getCurrentInstance();
    loginPath_ = epinfo.getContextPath() + "/faces/private/portal.jsp";
    
    userNameInput_ = new UIStringInput("userName", "");
    passwordInput_ = new UIStringInput("password", "");
    passwordInput_.setType(UIStringInput.PASSWORD);
    add(new Row().
        add(new LabelCell("#{UILogin.label.user-name}")).
        add(new ComponentCell(this, userNameInput_)));
    add(new Row().
        add(new LabelCell("#{UILogin.label.password}")).
        add(new ComponentCell(this, passwordInput_)));
    add(new Row().
        add(new ListComponentCell().
            add(new FormButton("#{UILogin.button.login}", LOGIN_ACTION)).            
            addColspan("2").addAlign("center")));
    addActionListener(LoginActionListener.class , LOGIN_ACTION) ;
  }

  public String getLoginPath() {  return loginPath_; }
  
  static public class LoginActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
		  UILogin comp = (UILogin) event.getComponent() ;     
			String userName = comp.userNameInput_.getValue() ;
			String password = comp.passwordInput_.getValue();
			InformationProvider iprovider = findInformationProvider(comp) ;
			boolean hasError = false ;
			if(userName == null || userName.length() == 0){
	      iprovider.addMessage(new ExoFacesMessage("#{UILogin.msg.empty-login}")) ;
	      hasError = true ;
	    }
	    if(password == null || password.length() == 0){
	      iprovider.addMessage(new ExoFacesMessage("#{UILogin.msg.empty-password}")) ;
	      hasError = true ;
	    }
	    if(hasError) return ;
	    if(comp.securityService_.authenticate(userName, password)) {
	    	ExternalContext eContext =comp.getFacesContext().getExternalContext();
				ActionResponse response = (ActionResponse) eContext.getResponse();
				response.addProperty(Output.LOGIN, userName);
				response.addProperty(Output.PASSWORD, password);
				response.sendRedirect(comp.loginPath_);
	    } else {
	      iprovider.addMessage(new ExoFacesMessage("#{UILogin.msg.unknow-user}")) ;
	    }
		}
	}
}