/*
 * Jul 8, 2004, 3:47:17 PM
 * @author: F. MORON
 * @email: francois.moron@laposte.net
 * 
 * */
package org.exoplatform.portal;

import java.util.ArrayList;
import java.util.regex.PatternSyntaxException;
import java.util.List;

import org.apache.commons.logging.Log;

import org.dom4j.*;
import org.dom4j.io.SAXReader;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.log.LogService;

public class ClientInfoMap {
  protected static Log log_;
  
  private ArrayList clientList_;
  private static ClientInfoMap singleton_;
  
  private void loadClientsInfos() {
    try {
      ClassLoader cl = Thread.currentThread().getContextClassLoader() ;
      java.net.URL url = cl.getResource("conf/portal/clients-infos.xml") ;
      SAXReader reader = new SAXReader();
      Document document = reader.read(url.openStream());
      List list = document.selectNodes("//clients-infos/client-infos");
      ClientInfo clientInfo;
      for(int i = 0; i <  list.size(); i++) {
        Node node = (Node) list.get(i) ;
        String name = node.selectSingleNode("name").getText() ;
        String userAgentPattern = node.selectSingleNode("userAgentPattern").getText() ;
        String preferredMimeType = node.selectSingleNode("preferredMimeType").getText() ;
        String renderer = node.selectSingleNode("renderer").getText() ;
        if (renderer.length() > 0) 
          clientInfo = new ClientInfo(name, userAgentPattern, preferredMimeType, renderer);
        else clientInfo = new ClientInfo(name, userAgentPattern, preferredMimeType);
        addClientInfo(clientInfo);
      } 
    } catch (Exception ex) {
      log_.error("Cannot load clients infos : " , ex) ;
    }
  }
  
  public ClientInfoMap() {
    clientList_ = new ArrayList();
    PortalContainer manager  = PortalContainer.getInstance();
    LogService service = (LogService) manager.getComponentInstanceOfType(LogService.class) ;
    log_ = service.getLog("org.exoplatform.portal");
    loadClientsInfos();
  }
  
  protected void addClientInfo(ClientInfo clientInfo) {
    clientList_.add(clientInfo);
  }
  
  /*
   * @return ClientInfo according to userAgent parameter and first
   * ClientInfo (ie5) if not found or if userAgent is null 
   * 
   */
  public ClientInfo findClient(String userAgent) {
    if (userAgent == null) return (ClientInfo) clientList_.get(0);
    if (userAgent.equals("")) return (ClientInfo) clientList_.get(0);
    ClientInfo client;
    for (int i = 0; i < clientList_.size(); i++) {
      client = (ClientInfo) clientList_.get(i);
      String userAgentPattern = client.getUserAgentPattern(); 
      if (userAgentPattern != null) {
        try {
          if (userAgent.matches(userAgentPattern)) return client;
        } catch (PatternSyntaxException e) {
          log_.debug("UserAgentPattern not valid :"+ userAgentPattern+" : "+e.getMessage());
          return (ClientInfo) clientList_.get(0);
        }
      }
    }
    
    return (ClientInfo) clientList_.get(0);
  }
  
  public static ClientInfoMap getInstance() {
    if (singleton_ == null) {
      synchronized (ClientInfoMap.class) {
        singleton_ = new ClientInfoMap();
      }
    }
    return singleton_;
  }
}
