package org.exoplatform.portal;

import java.util.*;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.context.PortletFacesContext;
import org.exoplatform.portal.faces.component.ChangeLocaleVisitor;
import org.exoplatform.portal.faces.component.UIPortal;
import org.exoplatform.portal.faces.event.ExoPortalEvent;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.services.portal.PortalConfigService;
import org.exoplatform.services.portal.model.Node;
import org.exoplatform.services.portal.model.PortalConfig;
import org.exoplatform.services.resources.LocaleConfig;
/**
 * Date: Aug 11, 2003
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: ExoPortalInfo.java,v 1.36 2004/10/27 15:08:53 tuan08 Exp $
 */
public class ExoPortalInfo implements org.exoplatform.portal.PortalInfo {
	final static public int VIEW_MODE = 0 ;
	final static public int EDIT_PAGE_MODE = 1 ;
	final static public int EDIT_PORTAL_MODE = 2 ;
	final static public int EDIT_NAVIGATION_MODE = 3 ;
	
	final static public String PRIVATE_PATH = "private";
	final static public String PUBLIC_PATH = "public";
	final static public String ADMIN_PATH = "admin";
	
  final public static String GROUP_TYPE = "group" ;
  final public static String USER_TYPE = "user" ;
  
  private String owner_ ;
  private String remoteUser_ ;
  private Map  ownerInfo_ ;
  private String portalType_ ;
  private String accessibility_  ;
  private ResourceBundle portalResource_ ;
  private ResourceBundle portalOwnerResource_ ;
  private LocaleConfig localeConfig_ ;
  private int portalMode_ ;
  private ClientInfo clientInfo_;
  private boolean new_ ;
  private PortalConfig portalConfig_ ;
  private String contextPath_ ;
  private String ipAddress_ ;
  private String ownerURI_ ;
  private String nodeURI_ ;
  private String viewId_ ;
  private PortalRequestParameters pparameters_ ;
  private boolean hasQueueEvent_ ;

  public ExoPortalInfo(String owner, String accessibility, HttpServletRequest request) {
    new_ = true ;
    owner_ = owner ;
    remoteUser_ = request.getRemoteUser() ;
    if(owner.startsWith("group:")) {
      portalType_ = GROUP_TYPE  ;
      ownerInfo_ = new HashMap() ;
    } else {
      portalType_ = USER_TYPE  ;
      ownerInfo_ = getUserInfoMap(owner) ;
    }
    accessibility_ = accessibility ;
    portalMode_ = VIEW_MODE ;
    contextPath_ = request.getContextPath() ;
    ownerURI_  = new StringBuffer().append(contextPath_).append("/faces/").
                 append(accessibility_).append("/").append(owner_).toString() ;
    if(request.getParameter("portletName") != null)  {
      viewId_ = "/page.jsp" ;
    } else  {
      viewId_ =  "/portal.jsp" ;
    }
    ipAddress_ = request.getRemoteAddr() ;
    setClientInfo(ClientInfoMap.getInstance().findClient(request.getHeader("User-Agent")));
    PortalContainer pcontainer = PortalContainer.getInstance() ;
    SessionContainer scontainer = pcontainer.createSessionContainer(request.getSession().getId()) ;
    scontainer.setPortalInfo(this) ;
    scontainer.getMonitor().setRemoteUser(request.getRemoteUser()) ;
    scontainer.getMonitor().setIPAddress(request.getRemoteAddr()) ;
  }

  public String getPortalOwner() { return owner_ ; } 
  public String getRemoteUser() { return remoteUser_ ; }
  public String getPortalType() { return portalType_ ; }
  public Map getOwnerInfoMap() { return ownerInfo_ ; }
  
  public String getPathInfo() { return nodeURI_ ;}

  public String getAccessibility() { return accessibility_ ; }
  
  public String getViewId() { return viewId_ ; }
  
  public ResourceBundle getPortalResourceBundle() { return portalResource_ ; }
  public void  setPortalResourceBundle(ResourceBundle res) { portalResource_ = res ;} 
  public ResourceBundle getApplicationResourceBundle() { return portalResource_ ; }
  
  public ResourceBundle getPortalOnwerResourceBundle() { return portalOwnerResource_ ; }
  public void  setPortalOwnerResourceBundle(ResourceBundle res) { portalOwnerResource_ = res ;} 
  
  public LocaleConfig getLocaleConfig() { return localeConfig_ ; }
  public void   setLocaleConfig(LocaleConfig config) {
    localeConfig_ = config ; 
  }
  
  public boolean isNew() { return new_ ; }
  public boolean setNew(boolean b) { return new_  = b; }
  
  public int getPortalMode() { return portalMode_ ; }
  public void  setPortalMode(int mode) { 
  	portalMode_ = mode ;
  	UIPortal uiPortal = getUIPortal() ;
  	if(mode == EDIT_PORTAL_MODE) uiPortal.setEditPortalMode() ;
  	else if(mode == EDIT_PAGE_MODE) uiPortal.setEditPageMode() ;
  	else if(mode == EDIT_NAVIGATION_MODE) uiPortal.setEditNavigationMode() ; 
  	else uiPortal.setViewPortalMode() ;
  }
  
  public String getContextPath() { return contextPath_ ; }
  
  public PortalConfig getPortalConfig()  throws Exception {
  	if(portalConfig_ != null) return portalConfig_ ;
  	PortalContainer manager  = PortalContainer.getInstance();
  	PortalConfigService configService = 
  		(PortalConfigService) manager.getComponentInstanceOfType(PortalConfigService.class) ;
  	portalConfig_ = configService.getPortalConfig(owner_) ; 
  	return portalConfig_  ;
  }
  
  public Map getUserInfoMap(String userName) {
  	try {
  		PortalContainer manager  = PortalContainer.getInstance();
  		OrganizationService service = 
  			(OrganizationService) manager.getComponentInstanceOfType(OrganizationService.class) ;
      UserProfile uP = service.findUserProfileByName(userName);
      if(uP ==null){
        return new HashMap() ;
      }
  		return uP.getUserInfoMap() ;
  	} catch (Exception ex) {
  		ex.printStackTrace() ;
  	}
  	return new HashMap() ;
  }
  
  
  public String getPageURI() {
    if(nodeURI_ == null) {
      String pageName = pparameters_.getPageName() ;
      if(pageName == null) {
        pageName = getUIPortal().getSelectedNode().getUri() ;
      }
      nodeURI_ = ownerURI_ + pageName ;
    }
    return nodeURI_ ; 
  }
  
  public String getOwnerURI() { return ownerURI_  ; }
  
  public PortalRequestParameters getPortalRequestParameters() { return pparameters_ ; }
  
  public void beginRequestLifecycle(PortalRequestParameters p) { 
    pparameters_ = p;
    hasQueueEvent_ = false ;
    nodeURI_ = null ;
  }
  
  public void endRequestLifecycle() {
    
  }
  
  public Node getRootNode()  { return getUIPortal().getRootNode() ; }
  
  public Node getSelectedNode()  { return getUIPortal().getSelectedNode() ; }
  
  public void setSelectedNode(Node  node)  throws Exception {
    getUIPortal().setSelectedNode(node) ;
    if(node != null) nodeURI_ = ownerURI_ + node.getUri() ;
  }
  
  private UIPortal getUIPortal() {
    FacesContext portalContext = FacesContext.getCurrentInstance() ;
    if (portalContext instanceof PortletFacesContext) {
      portalContext = ((PortletFacesContext) portalContext).getPortalFacesContext() ;   
    }
    return (UIPortal) portalContext.getViewRoot().getChildren().get(0) ;
  }

	public ClientInfo getClientInfo() { return clientInfo_; }
	public void setClientInfo(ClientInfo clientInfo) { clientInfo_ = clientInfo; }

	public String getPreferredMimeType() { return clientInfo_.getPreferredMimeType() ; }
	public String getClientName() {  return clientInfo_.getName() ; }
  
	public String getClientIPAddress() { return ipAddress_ ;} 
 
  public ExoPortalEvent createExoPortalEvent(String action) {
    UIPortal uiPortal = getUIPortal() ;
    ExoPortalEvent event = new ExoPortalEvent(uiPortal, action) ;
    return event ;
  }
  
  public void queuePortalEvent(ExoPortalEvent event) {
    getUIPortal().queueEvent(event) ;
    hasQueueEvent_ = true ;
  }
  
  public boolean hasQueueEvent() { return hasQueueEvent_ ; }
  
  public void changeLocale(String localeName) {
    UIPortal uiPortal = getUIPortal() ;
    new ChangeLocaleVisitor().changeLocale(uiPortal, localeName) ;
    new ChangeLocaleVisitor().changeLocale(uiPortal.getCurrentUIPage()) ;
  }
  
  static public ExoPortalInfo getCurrentInstance() {
  	SessionContainer scontainer = SessionContainer.getInstance() ;
    return (ExoPortalInfo) scontainer.getPortalInfo() ;
  }
}