/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;

import org.exoplatform.commons.utils.MapResourceBundle;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.ComponentVisitor;
import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.portal.ExoPortalInfo;
import org.exoplatform.portal.PortalConstants;
import org.exoplatform.services.resources.LocaleConfig;
import org.exoplatform.services.resources.LocaleConfigService;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Nov 17, 2004
 * @version $Id$
 */
public class ChangeLocaleVisitor extends ComponentVisitor {
  private ResourceBundle res_ ;
  
  public ChangeLocaleVisitor() {
    
  }
  
  /* set default locale */
  public void changeLocale(UIPortal uiPortal) {
    String lang = uiPortal.getPortalConfigModel().getLocale() ;
    FacesContext context = FacesContext.getCurrentInstance() ;
    HttpServletRequest request = 
      (HttpServletRequest) context.getExternalContext().getRequest();
    String langParam = request.getParameter(PortalConstants.LANGUAGE_PARAMETER) ;
    if(langParam != null) lang = langParam ;
    if (lang == null || lang.length() == 0) {
      Locale requestLocale = request.getLocale() ;
      lang = requestLocale.getLanguage() ;
    }
    changeLocale(uiPortal, lang) ;
  }
  
  public void changeLocale(UIPortal uiPortal, String locale) {
    LocaleConfigService manager = 
      (LocaleConfigService)PortalContainer.getInstance().
                           getComponentInstanceOfType(LocaleConfigService.class) ;
    LocaleConfig localeConfig = manager.getLocaleConfig(locale);
    ExoPortalInfo epinfo = ExoPortalInfo.getCurrentInstance() ;
    epinfo.setLocaleConfig(localeConfig) ;
    ResourceBundle rB = localeConfig.getResourceBundle("locale.portal.portal");
    MapResourceBundle outputBundled = new MapResourceBundle(rB, new Locale(locale));
    outputBundled.resolveDependencies();
    epinfo.setPortalResourceBundle(outputBundled) ;
    epinfo.setPortalOwnerResourceBundle(localeConfig.getOwnerResourceBundle(epinfo.getPortalOwner())) ;
    res_ = epinfo.getPortalOnwerResourceBundle() ;
    visit(uiPortal) ;
    traverse(uiPortal.getRootContainer());
  }
  
  public void changeLocale(UIPage uiPage) {
    ExoPortalInfo epinfo = ExoPortalInfo.getCurrentInstance() ;
    LocaleConfig config = epinfo.getLocaleConfig();
    res_ = config.getOwnerResourceBundle(uiPage.getOwner()) ;
    traverse(uiPage) ;
  }
  
  public void changeLocale(UISinglePage comp, String locale) {
    LocaleConfigService manager = 
      (LocaleConfigService)PortalContainer.getInstance().
                           getComponentInstanceOfType(LocaleConfigService.class) ;
    LocaleConfig localeConfig = manager.getLocaleConfig(locale);
    ExoPortalInfo epinfo = ExoPortalInfo.getCurrentInstance() ;
    epinfo.setLocaleConfig(localeConfig) ;
    ResourceBundle rB = localeConfig.getResourceBundle("locale.portal.portal");
    MapResourceBundle outputBundled = new MapResourceBundle(rB, new Locale(locale));
    outputBundled.resolveDependencies();
    epinfo.setPortalResourceBundle(outputBundled) ;
    epinfo.setPortalOwnerResourceBundle(localeConfig.getOwnerResourceBundle(epinfo.getPortalOwner())) ;
    res_ = epinfo.getPortalOnwerResourceBundle() ;
    List children = comp.getChildren() ;
    for(int i = 0; i < children.size() ; i++) {
      UIExoComponent child = (UIExoComponent) children.get(i);
      visit(child) ;
    }
  }
  
  public  void traverse(UIExoComponent component)  {
    visit(component) ;
    if(component instanceof UIContainer) {
      List children = component.getChildren() ;
      for(int i = 0; i < children.size() ; i++) {
        UIExoComponent child = (UIExoComponent) children.get(i);
        traverse(child) ;
      }
    }
  }
  
  public void visit(UIExoComponent comp) {
    if(comp instanceof UIBasicComponent) {
      UIBasicComponent uiComp = (UIBasicComponent) comp ;
      uiComp.changeLocale(res_) ;
    }
  }
}