/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import org.exoplatform.commons.utils.ExceptionUtil;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.UIComponentFactory;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.portal.ExoPortalInfo;
import org.exoplatform.portal.PortalConstants;
import org.exoplatform.portal.PortalRequestParameters;
import org.exoplatform.portal.faces.listener.page.PageActionListener;
import org.exoplatform.portal.faces.listener.portal.NodeActionListener;
import org.exoplatform.portal.faces.listener.portal.PortalActionListener;
import org.exoplatform.services.portal.PortalACL;
import org.exoplatform.services.portal.PortalConfigService;
import org.exoplatform.services.portal.model.*;
import org.exoplatform.services.portal.skin.SkinConfigService;
import org.exoplatform.services.portal.skin.model.Style;
import org.exoplatform.services.resources.LocaleConfig;
/**
 * Date: Aug 11, 2003
 * @author : Mestrallet Benjamin
 * @email:   benjmestrallet@users.sourceforge.net
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIPortal.java,v 1.36 2004/10/27 02:53:05 tuan08 Exp $
 */
public class UIPortal extends UIBasicComponent {
  public static final String DEFAULT_RENDERER_TYPE = "PortalRenderer";
  
  private String userCss_ ;
  private Node root_ ;
  private Node selectNode_ ;
  private Map componentCache_ ;
  private boolean modified_ ;
  private PortalConfigService configService_ ;
  private ExoPortalInfo epInfo_ ;
  private PortalConfig componentModel_ ;
  private boolean dirty_ = false ;
  private UIComponentFactory componentFactory_ ;
  private PortalACL portalACL_ ;
  
	public UIPortal() throws Exception {
    try {
    epInfo_ = ExoPortalInfo.getCurrentInstance();
    componentModel_ =  epInfo_.getPortalConfig() ;
    initBasicComponent(componentModel_, "default") ;
    componentCache_ = new HashMap(20) ; 
    PortalContainer pcontainer  = PortalContainer.getInstance();
    configService_ = 
      (PortalConfigService) pcontainer.getComponentInstanceOfType(PortalConfigService.class) ;
    portalACL_ = configService_.getPortalACL() ;
    List children = getChildren() ;
    
    root_ = configService_.getNodeNavigation(getOwner())  ;
    root_.setVisible(portalACL_, epInfo_.getPortalOwner(), epInfo_.getRemoteUser()) ;
    String selectNodeURI = epInfo_.getPortalRequestParameters().getPageName() ;
    selectNode_ = root_.findNode(selectNodeURI) ;
    if(selectNode_ == null) selectNode_ = root_ ;
    Container layout = componentModel_.getLayout() ;
    if(ExoPortalInfo.XHTMLMP_MIME_TYPE.equals(epInfo_.getPreferredMimeType())) {
      layout = componentModel_.getMobilePortalLayout() ;
    }
    UIContainer uiPortalLayout = new UIContainer(layout, "default", null);
		children.add(uiPortalLayout) ;
    new ChangeLocaleVisitor().changeLocale(this) ;
   
    if(ExoPortalInfo.PRIVATE_PATH.equals(epInfo_.getAccessibility())){
      setComponentAdminRole(true) ;   
    } else if(ExoPortalInfo.ADMIN_PATH.equals(epInfo_.getAccessibility())){
    	setComponentAdminRole(true) ;   
    } else {
      setComponentAdminRole(false) ;   
    }
    registerListeners() ;
    UIBody uiBody = findUIBody(uiPortalLayout) ;
    uiBody.initDefaultBodyComponent() ;
    userCss_ = CssStyleBuilderVisitor.getCSS(getRootContainer()) ;
    } catch (Throwable t) { t.printStackTrace() ;}
	}
  
  private void registerListeners() {
    addActionListener(NodeActionListener.class, PortalConstants.EDIT_NODE_ACTION) ;
    addActionListener(NodeActionListener.class, PortalConstants.ADD_NODE_ACTION) ;
    addActionListener(NodeActionListener.class, PortalConstants.DELETE_NODE_ACTION) ;
    addActionListener(NodeActionListener.class, PortalConstants.SAVE_NODE_ACTION) ;
    addActionListener(NodeActionListener.class, PortalConstants.BROWSE_PAGE_ACTION) ;
    addActionListener(NodeActionListener.class, PortalConstants.MOVE_UP_NODE_ACTION) ;
    addActionListener(NodeActionListener.class, PortalConstants.MOVE_DOWN_NODE_ACTION) ;
    
    addActionListener(PageActionListener.class, PortalConstants.SAVE_PAGE_ACTION) ;
    
    addActionListener(PortalActionListener.class, PortalConstants.EDIT_PORTAL_ACTION) ;
    addActionListener(PortalActionListener.class, PortalConstants.SAVE_PORTAL_ACTION) ;
  }
    
  public String getFamily() { return "org.exoplatform.portal.faces.component.UIPortal" ; }
  protected String getSkinName() { return "default" ; } 
  protected String getIdPrefix() { return "portal" ; }
  
  public LocaleConfig getLocaleConfig() { return epInfo_.getLocaleConfig() ; }
  
  public Component getComponentModel() { return componentModel_ ; }
  public PortalConfig getPortalConfigModel() { return componentModel_ ; }
  public PortalConfig getEditablePortalConfigModel() {
  	if(!modified_) {
  		componentModel_ = (PortalConfig)componentModel_.softCloneObject() ;
  	}
  	return componentModel_ ;
  }
  
  public String getOwner() { return epInfo_.getPortalOwner() ; }
  public String getContextPath() { return epInfo_.getAccessibility() ; }

  public String getUserCss() { return userCss_ ; } 
  
  protected String getDefaultRendererType() { return DEFAULT_RENDERER_TYPE ; }

  /**Override default set editable in UIBasicComponent*/
  //public void setComponentAdminRole(boolean b) { componentAdminRole_ = b ; }
  
  public boolean isDirty() { return dirty_ ; }
  /**Override default setDirty in UIBasicComponent*/
  public void setDirty(boolean b) { dirty_ = b ; }
  
  public void setViewPortalMode() {
    setSelectedNode(selectNode_) ;
    setComponentMode(UIBasicComponent.COMPONENT_VIEW_MODE);
    findUIBody().setBodyMode(UIBasicComponent.COMPONENT_VIEW_MODE);
  }
  
  public boolean hasEditPortalCapability() { 
    return hasComponentAdminRole() && 
           portalACL_.hasEditPortalPermission(componentModel_, getOwner()) ;
  }
  
  public void setEditPortalMode() {
  	if(hasEditPortalCapability()) {
  		setComponentMode(UIBasicComponent.COMPONENT_EDIT_MODE);
  		findUIBody().setBodyMode(UIBasicComponent.COMPONENT_VIEW_MODE);
  		mode_ = UIBasicComponent.COMPONENT_VIEW_MODE ;
  	}
  }
  
  public void setEditPageMode() {
    setComponentMode(UIBasicComponent.COMPONENT_VIEW_MODE);
    setSelectedNode(getSelectedNode()) ;
    findUIBody().setBodyMode(UIBasicComponent.COMPONENT_EDIT_MODE) ;
  }
  
  public boolean hasEditNavigationCapability() { 
    return hasComponentAdminRole() && 
           portalACL_.hasEditNavigationPermission(componentModel_.getEditPermission(), 
                                                  componentModel_.getOwner(), getOwner());
  }
  
  public void setEditNavigationMode() {
  	if(hasEditNavigationCapability()) {
  		setComponentMode(UIBasicComponent.COMPONENT_VIEW_MODE);
  		findUIBody().setBodyMode(UIBasicComponent.COMPONENT_VIEW_MODE);
  		mode_ = UIBasicComponent.COMPONENT_EDIT_MODE ;
  	}
  }
  
  public Node  getRootNode() { return root_ ; }
  public Node  getSelectedNode() { return selectNode_ ; }
  public void  setSelectedNode(Node node) {
    if(selectNode_!= null) {
      selectNode_.setSelectedPath(false) ;
      selectNode_ = node ;
      selectNode_.setSelectedPath(true) ;
      UIPage uiPage = getUIPage(selectNode_) ;
      new ChangeLocaleVisitor().changeLocale(uiPage) ;
      setBodyComponent(uiPage) ;
    }
  }
  
  public void  removeSelectedNode() {
    Node parent = selectNode_.getParent() ;
    if (parent != null && !selectNode_.isShare()) {
      parent.removeChild(selectNode_.getUri()) ;
      componentCache_.get(selectNode_.getPageReference()) ;
      setSelectedNode(parent) ;
    }
  }
  
  public Object getPortalComponent(Class clazz) {
  	Object object  =  componentCache_.get(clazz.getName()) ;
  	if(object == null) {
  		try {
  			object =  componentFactory_.createUIComponent(clazz) ;
  			componentCache_.put(clazz.getName(), object)  ;
  		} catch (Exception ex) { 
  			log_.error("create component :" + clazz.getName(), ex) ;
  		}
  	}
  	return object ;
  }
  
  public UIBody findUIBody(UIContainer container) {
    List children = container.getChildren();
    int childrenSize = children.size() ;
    UIBody uiBody = null ;
    for(int i = 0; i < childrenSize; i++) {
      Object child = children.get(i) ;
      if(child instanceof UIBody) {
        uiBody = (UIBody) child ;
      } else if (child instanceof UIContainer) {
        uiBody = findUIBody((UIContainer) child) ;
      }
      if(uiBody != null) return uiBody ;
    }
    return null ;
  }
  
  final public UIBody findUIBody() { return findUIBody(getRootContainer()) ; }
  
  final public UIContainer getRootContainer() { return (UIContainer) getChildren().get(0) ; }
  
  public UIPage getUIPage(Node node) {
    PageReference pref = node.getPageReference(epInfo_.getPreferredMimeType()) ;
    if(pref == null ) {
      pref = node.getPageReference(ExoPortalInfo.XHTML_MIME_TYPE) ;
    }
    String referenceId = pref.getPageReference();
    UIPage uiPage = (UIPage)  componentCache_.get(referenceId) ;
    if (uiPage == null && referenceId != null) {
			try {
				Page config = configService_.getPage(referenceId);
				if(config == null) {
				  config = configService_.getPage(PortalConfigService.PAGE_NOT_FOUND_ERROR);
				}
				if (!portalACL_.hasViewPagePermission(config, getOwner())) {
					config = configService_ .getPage(PortalConfigService.PAGE_NOT_FOUND_ERROR);
					pref.setPageReference(PortalConfigService.USER_PAGE_VIEW_PERMISSION_ERROR);
				  setModified(true);
				}
				uiPage = new UIPage(config, "default");
				uiPage.setRendered(true);
				componentCache_.put(referenceId, uiPage);
        if(hasComponentAdminRole() && 
           portalACL_.hasEditPagePermission(config, getOwner())) {
        	uiPage.setComponentAdminRole(true) ;
        } 
			} catch (Throwable t) {
			  t = ExceptionUtil.getRootCause(t) ;
				log_.error("Error: ", t);
			}
		}
    return uiPage ;
  }
  
  public UIPage getCurrentUIPage() { return getUIPage(selectNode_) ; }
  
  public void setBodyComponent(UIComponent component) { 
    if(component == null) {
      log_.warn("set body component, component is null") ;
      return ;
    }
    UIContainer uiLayout = (UIContainer)getChildren().get(0) ;
    UIBody uiBody = findUIBody(uiLayout) ;
    uiBody.setBodyComponent(component) ;
    if(epInfo_.getPortalMode() == ExoPortalInfo.EDIT_PAGE_MODE) {
      uiBody.setBodyMode(UIBasicComponent.COMPONENT_EDIT_MODE) ;
    } else {
      uiBody.setBodyMode(UIBasicComponent.COMPONENT_VIEW_MODE) ;
    }
    userCss_ = CssStyleBuilderVisitor.getCSS(getRootContainer()) ;
  }
  
  public void setLastBodyComponent() { 
    UIContainer uiLayout = (UIContainer)getChildren().get(0) ;
    UIBody uiBody = findUIBody(uiLayout) ;
    uiBody.setLastBodyComponent() ;
    userCss_ = CssStyleBuilderVisitor.getCSS(getRootContainer()) ;
  }
  
  public void removeUIPageFromCache(String referenceId) {
    componentCache_.remove(referenceId);
  }
 
  final public void processDecodes(FacesContext context) {
    PortalRequestParameters pparams = epInfo_.getPortalRequestParameters();
		String targetComponent = pparams.getTargetComponentId();
		if (targetComponent != null) {
			UIComponent uiTargetComponent = (UIComponent)findComponentById(targetComponent);
      if(uiTargetComponent == null) {
        //log_.info("Cannot find the the target component id: " + targetComponent) ; 
      } else {
        uiTargetComponent.processDecodes(context);
      }
		} else {
			UIBody uiBody = findUIBody((UIContainer) getChildren().get(0));
			uiBody.processDecodes(context);
		}
  }
  
  final public  void processValidators(FacesContext context) {
    PortalRequestParameters pparams = epInfo_.getPortalRequestParameters();
    if(pparams.getPortalAction() != null)  {
      super.processValidators(context) ;
    }
  }

  protected String getComponentIdPrefix() { return "" ; }
  protected Style getDecoratorStyle(SkinConfigService service, String renderer, String style) {
    return null ;
  }
  
  public void changeLocale(ResourceBundle res) {
    componentFactory_ = new UIComponentFactory(res) ;
    root_.visit(new ResolveLabelVisitor(res)) ;
  }
  
  public void buildTreeModel(Container parent) { }
  
  class ResolveLabelVisitor implements NodeVisitor {
    ResourceBundle res_ ;
    
    public ResolveLabelVisitor(ResourceBundle res) {    res_ = res ;   }
    
    public void visit(Node node) {    node.setResolvedLabel(res_) ;   }   
  }
}