/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.filter;

import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.commons.logging.Log;
import org.exoplatform.commons.utils.ExceptionUtil;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.RootContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.container.monitor.ActionData;
import org.exoplatform.portal.ExoPortalInfo;
import org.exoplatform.portal.PortalRequestParameters;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.HibernateServiceContainer;
import org.exoplatform.services.log.LogService;
/**
 * Fri, May 30, 2003 @
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: PrivateRequestFilter.java,v 1.25 2004/11/03 01:19:46 tuan08 Exp $
 */
public class AdminRequestFilter implements Filter {
  final public static String ACTION = "action" ;
  final public static String ADMIN = "admin" ;
  final public static String RETURN = "return" ;
 
  private Log log_ ;
  private String portalName_ ;
  private HibernateServiceContainer hserviceContainer_;
	
	public void init(FilterConfig filterConfig) {
    portalName_ = filterConfig.getServletContext().getServletContextName() ;
    PortalContainer pcontainer = RootContainer.getInstance().getPortalContainer(portalName_) ;
    PortalContainer.setInstance(pcontainer) ;		
    LogService lservice = (LogService) pcontainer.getComponentInstanceOfType(LogService.class) ; 
    log_  = lservice.getLog("org.exoplatform.portal.filter") ;
    PortalContainer.setInstance(null) ;
	}
    
  public void doFilter(ServletRequest request, ServletResponse response, 
                       FilterChain chain) throws IOException, ServletException {
   
    long start = System.currentTimeMillis() ;
    PortalContainer pcontainer = RootContainer.getInstance().getPortalContainer(portalName_) ;
    PortalContainer.setInstance(pcontainer) ;
    HttpServletRequest httpRequest = (HttpServletRequest) request ;
    HttpSession session = httpRequest.getSession() ; 
    String action = httpRequest.getParameter(ACTION) ;
    if(ADMIN.equals(action)) {
      saveAdminPortal(httpRequest,pcontainer)  ;
    } else if(RETURN.equals(action)) {
      HttpServletResponse httpResponse = (HttpServletResponse) response ;
      restoreAdminPortal(httpRequest, httpResponse, pcontainer) ;
      return ;
    }
    SessionContainer scontainer = (SessionContainer)pcontainer.getComponentInstance(session.getId()) ;
    ExoPortalInfo epinfo =  null ;
    if(scontainer != null) {
      epinfo =(ExoPortalInfo) scontainer.getPortalInfo() ;
      epinfo.getLocaleConfig().setInput(httpRequest) ;
    }
    PortalRequestParameters pparameters = null ;
    try {
      pparameters = new PortalRequestParameters(httpRequest) ;
      String owner = pparameters.getPortalOwner() ;
      if (epinfo == null) {
        epinfo =  new ExoPortalInfo(owner, ExoPortalInfo.ADMIN_PATH, httpRequest) ;
        scontainer = (SessionContainer)pcontainer.getComponentInstance(session.getId()) ;
        pparameters = new PortalRequestParameters(epinfo.getContextPath(), owner, httpRequest) ;
      } else {
        pparameters = new PortalRequestParameters(httpRequest) ;
      }
      epinfo.beginRequestLifecycle(pparameters) ;
      scontainer.startActionLifcycle() ;
      httpRequest.setAttribute("javax.servlet.include.path_info", epinfo.getViewId()) ;
      chain.doFilter(request, response) ;
      epinfo.setNew(false) ;
    } catch (Throwable ex) {
      ex = ExceptionUtil.getRootCause(ex) ;
      log_.error("Error: ", ex) ;
    } finally {
      getHibernateServiceContainer(pcontainer).closeAllSessions() ;
      long end = System.currentTimeMillis()  ;
      ActionData data = new ActionData(pparameters.getPortalOwner(), 
                                       pparameters.getPageName(),  
                                       httpRequest.getMethod(), 
                                       end - start, 
                                       httpRequest.getParameterMap()) ;
      scontainer.getMonitor().log(data) ;
      scontainer.endActionLifcycle() ;
      PortalContainer.setInstance(null) ;
    }
  }
  
  private void saveAdminPortal(HttpServletRequest httpRequest, 
                               PortalContainer pcontainer) {
    HttpSession session = httpRequest.getSession() ;
    SessionContainer scontainer = (SessionContainer)pcontainer.getComponentInstance(session.getId()) ;
    ExoPortalInfo epinfo =(ExoPortalInfo) scontainer.getPortalInfo() ;
    Object jsfTree = session.getAttribute(epinfo.getViewId()) ;
    Backup backup = new Backup(scontainer, jsfTree) ; 
    session.setAttribute("org.exoplatform.portal.backup", backup) ;
    pcontainer.removeSessionContainer(session.getId()) ;
    session.removeAttribute(epinfo.getViewId()) ;
  }
  
  private void restoreAdminPortal(HttpServletRequest request,
                                  HttpServletResponse response,
                                  PortalContainer pcontainer) throws IOException {
    HttpSession session = request.getSession() ;
    Backup backup = (Backup)session.getAttribute("org.exoplatform.portal.backup") ;
    session.removeAttribute("org.exoplatform.portal.backup") ;
    SessionContainer scontainer = backup.scontainer_ ;
    ExoPortalInfo epinfo =(ExoPortalInfo) scontainer.getPortalInfo() ;
    pcontainer.removeSessionContainer(session.getId()) ;
    pcontainer.registerComponentInstance(session.getId(), scontainer) ;
    session.setAttribute(epinfo.getViewId(),  backup.jsfTree_) ;
    response.sendRedirect(epinfo.getPageURI()) ;
  }
  
  public void destroy() { }
  
  private HibernateServiceContainer getHibernateServiceContainer(PortalContainer pcontainer){
    if(hserviceContainer_ == null){
      hserviceContainer_ = (HibernateServiceContainer) pcontainer.getComponentInstanceOfType(
          HibernateServiceContainer.class) ;
    }
    return hserviceContainer_;
  }    
  
  static class Backup {
    SessionContainer scontainer_ ;
    Object jsfTree_ ;
    
    public Backup(SessionContainer scontainer, Object jsfTree) {
      scontainer_ = scontainer;
      jsfTree_ = jsfTree ;
    }
  }
}