/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.filter;

import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.commons.logging.Log;
import org.exoplatform.commons.utils.ExceptionUtil;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.RootContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.container.monitor.ActionData;
import org.exoplatform.portal.Constants;
import org.exoplatform.portal.ExoPortalInfo;
import org.exoplatform.portal.PortalRequestParameters;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.HibernateServiceContainer;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.portal.PortalACL;
import org.exoplatform.services.portal.model.PortalConfig;
/*
 * Fri, May 30, 2003 @
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: PublicRequestFilter.java,v 1.28 2004/11/03 01:19:46 tuan08 Exp $
 */
public class PublicRequestFilter implements Filter {
  private Log log_ ;  
  private PortalACL portalACL_ ;
  private String portalName_ ;
  private HibernateServiceContainer hserviceContainer_;
  
  public void init(FilterConfig filterConfig) {
    portalName_ = filterConfig.getServletContext().getServletContextName() ;
    PortalContainer pcontainer = RootContainer.getInstance().getPortalContainer(portalName_) ;
    PortalContainer.setInstance(pcontainer) ;    
    portalACL_ = (PortalACL) pcontainer.getComponentInstanceOfType(PortalACL.class) ;
    LogService lservice = (LogService) pcontainer.getComponentInstanceOfType(LogService.class) ; 
    log_  = lservice.getLog("org.exoplatform.portal.filter") ;
    PortalContainer.setInstance(null) ;
  }

  public void doFilter(ServletRequest request, ServletResponse response, 
                       FilterChain chain) throws IOException, ServletException {
    long start = System.currentTimeMillis() ;
    PortalContainer pcontainer = RootContainer.getInstance().getPortalContainer(portalName_) ;
    PortalContainer.setInstance(pcontainer) ;
    HttpServletRequest httpRequest = (HttpServletRequest) request ;
    HttpSession session = session = httpRequest.getSession() ;     
    SessionContainer scontainer = (SessionContainer)pcontainer.getComponentInstance(session.getId()) ;
    ExoPortalInfo epinfo = null ;
    if(scontainer != null) {
      epinfo =(ExoPortalInfo) scontainer.getPortalInfo() ;
      epinfo.getLocaleConfig().setInput(httpRequest) ;
    }
    PortalRequestParameters pparameters = null ;
    try {
      //if user login , invalidate the session to logout when user move to public link
      pparameters = new PortalRequestParameters(httpRequest) ;
      String owner = pparameters.getPortalOwner() ;
      if (epinfo == null  || 
          epinfo.getAccessibility() != ExoPortalInfo.PUBLIC_PATH ||
         (owner != null && !epinfo.getPortalOwner().equals(owner))) {
        if (owner == null) owner = Constants.DEFAUL_PORTAL_OWNER ;
        Util.removeAttribute(session) ;
        epinfo = new ExoPortalInfo(owner, ExoPortalInfo.PUBLIC_PATH, httpRequest) ;
        PortalConfig config = epinfo.getPortalConfig() ;
        if(!portalACL_.hasViewPortalPermission(config, null)) {
          HttpServletResponse httpResponse = (HttpServletResponse) response ;
          pcontainer.removeSessionContainer(session.getId()) ;
          httpResponse.sendRedirect(epinfo.getContextPath() + "/access-error.jsp") ;
          return ;
        } 
        scontainer = (SessionContainer)pcontainer.getComponentInstance(session.getId()) ;
      }
      epinfo.beginRequestLifecycle(pparameters) ;
      scontainer.startActionLifcycle(); 
      httpRequest.setAttribute("javax.servlet.include.path_info", epinfo.getViewId()) ;
      chain.doFilter(request, response) ;
      epinfo.setNew(false) ;
    } catch (Throwable ex) {
      ex = ExceptionUtil.getRootCause(ex) ;
      log_.error("Error: ", ex) ;
    } finally { 
      getHibernateServiceContainer(pcontainer).closeAllSessions() ;
      long end = System.currentTimeMillis() ;
      ActionData data = new ActionData(pparameters.getPortalOwner(), 
                                       pparameters.getPageName(), 
                                       httpRequest.getMethod(), 
                                       end - start, 
                                       httpRequest.getParameterMap()) ;
      scontainer.getMonitor().log(data) ;
      if(scontainer != null)scontainer.endActionLifcycle() ;
      PortalContainer.setInstance(null) ;
    }
  }
  
  public void destroy() { }
  
  private HibernateServiceContainer getHibernateServiceContainer(PortalContainer pcontainer){
    if(hserviceContainer_ == null){
      hserviceContainer_ = (HibernateServiceContainer) pcontainer.getComponentInstanceOfType(
          HibernateServiceContainer.class) ;
    }
    return hserviceContainer_;
  }        
}