/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.communication.forum.component;

import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.communication.forum.Category;
import org.exoplatform.services.communication.forum.Forum;
import org.exoplatform.services.communication.forum.ForumService;

/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIForumForm.java,v 1.7 2004/10/17 13:46:25 tuan08 Exp $
 */
public class UIForumForm extends UISimpleForm {
  private ForumService service_ ;
  private UIStringInput forumName_ ;
  private UITextArea    description_ ;
  private UIStringInput viewForumRole_ ;
  private UIStringInput createTopicRole_ ;
  private UIStringInput replyTopicRole_ ;
  private UIStringInput moderators_ ;
  private Forum forum_ ;
  private String categoryId_ ;
  
  public UIForumForm(ForumService service) throws Exception {
    super("forumForm", "post", null) ;
    setId("UIForumForm") ;
    setClazz("UIForumForm") ;
    service_ = service ;
    description_ = new UITextArea("description", "") ;
    forumName_ = new UIStringInput("forumName", "") ;
    viewForumRole_ = new UIStringInput("viewForumRole", "") ;
    createTopicRole_ = new UIStringInput("createTopicRole", "") ;
    replyTopicRole_ = new UIStringInput("replyTopicRole", "") ;
    moderators_ = new UIStringInput("moderators", "") ;

    add(new HeaderRow().
        add(new Cell("#{UIForumForm.header.title}").
            addColspan("2")));
    add(new Row().
        add(new LabelCell("#{UIForumForm.label.name}")).
        add(new ComponentCell(this, forumName_)));
    add(new Row().
        add(new LabelCell("#{UIForumForm.label.description}").
            addValign("top")).
        add(new ComponentCell(this, description_)));
    add(new Row().
        add(new LabelCell("#{UIForumForm.label.view-forum-role}")).
        add(new ComponentCell(this, viewForumRole_)));
    add(new Row().
        add(new LabelCell("#{UIForumForm.label.create-topic-role}")).
        add(new ComponentCell(this, createTopicRole_)));
    add(new Row().
        add(new LabelCell("#{UIForumForm.label.reply-topic-role}")).
        add(new ComponentCell(this, replyTopicRole_)));
    add(new Row().
        add(new LabelCell("#{UIForumForm.label.moderators}")).
        add(new ComponentCell(this, moderators_)));
    add(new Row().
        add(new ListComponentCell().
            add(new FormButton("#{UIForumForm.button.save}", SAVE_ACTION)).
            add(new FormButton("#{UIForumForm.button.cancel}", CANCEL_ACTION)).
            addColspan("2").addAlign("center"))) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION)  ;
    addActionListener(SaveActionListener.class, SAVE_ACTION)  ;
  }
  
  public void setForum(String categoryId, Forum forum) {
    forum_ = forum ;
    categoryId_ = categoryId ;
    if (forum == null) {
      forumName_.setText("") ;
      description_.setText("") ;
      viewForumRole_.setText("") ;
      createTopicRole_.setText("") ;
      replyTopicRole_.setText("") ;
      moderators_.setText("") ;
    } else {
      forumName_.setText(forum.getForumName()) ;
      description_.setText(forum.getDescription()) ;
      viewForumRole_.setText(forum.getViewForumRole()) ;
      createTopicRole_.setText(forum.getCreateTopicRole()) ;
      replyTopicRole_.setText(forum.getReplyTopicRole()) ;
      moderators_.setText(forum.getModerators()) ;
    }
  }
  
  public boolean canDecodeInvalidState()  { return categoryId_ != null; }
  
  static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIForumForm uiForm = (UIForumForm) event.getComponent() ;
			uiForm.setRenderedSibling(UIAdminViewCategories.class) ;
		}
  }
  
  static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIForumForm uiForm = (UIForumForm) event.getComponent() ;
			Forum forum = uiForm.forum_ ;
      boolean isNew =  false ;
      if(forum == null) {
      	forum = uiForm.service_.createForumInstance() ;
        isNew = true ;
      }
      forum.setForumName(uiForm.forumName_.getValue()) ;
      forum.setDescription(uiForm.description_.getValue()) ;
      forum.setViewForumRole(uiForm.viewForumRole_.getValue()) ;
      forum.setCreateTopicRole(uiForm.createTopicRole_.getValue()) ;
      forum.setReplyTopicRole(uiForm.replyTopicRole_.getValue()) ;
      forum.setModerators(uiForm.moderators_.getValue()) ;
      if(isNew) {
        Category category = uiForm.service_.getCategory(uiForm.categoryId_) ;
        uiForm.service_.addForum(category , forum) ;
      } else {
        uiForm.service_.updateForum(forum) ;
      }
      UIAdminViewCategories uiCategories =
       	(UIAdminViewCategories) uiForm.getSibling(UIAdminViewCategories.class) ;
      uiCategories.update() ;
			uiForm.setRenderedSibling(UIAdminViewCategories.class) ;
		}
  }
}