/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.explorer.component;

import java.io.File;
import java.io.FileInputStream;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;
import javax.faces.render.Renderer;
import org.exoplatform.commons.utils.MimeTypeUtil;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIFileUpload;
import org.exoplatform.faces.core.component.UIPortlet;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.ACL;
import org.exoplatform.portlets.content.explorer.component.model.NodeDescriptor;

/**
 * Jun 22, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIFileExplorer.java,v 1.3 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UIFileExplorer extends UIExplorer {
	private Map cacheNodes_ = new WeakHashMap() ;
	private String baseDir_ ;
	private ACL acl_ ;   
	
	public UIFileExplorer() {
	  addActionListener(ChangeNodeActionListener.class, CHANGE_NODE) ;
	  addActionListener(RemoveNodeActionListener.class, REMOVE_NODE) ;
	  addActionListener(EditContentActionListener.class, EDIT_NODE_CONTENT) ;
	  addActionListener(ViewContentActionListener.class, VIEW_NODE_CONTENT) ;
	  addActionListener(UploadFileActionListener.class, UPLOAD_FILE) ;
	}
	
	public void init(ACL acl, String baseDir) throws Exception {
		acl_ = acl ;
		baseDir_ = baseDir ;
		changeNode("/") ;
	}
	
	/**
   * This method check the NodeDescriptor in the cache. if the descriptor is not found
   * then build an UINodeDescriptor, its children and cache it. Note all the users will share the
   * UINodeDescriptor objects in the cache. It will save a lot of memory and cpu. If a 
   * user delete a folder node the whole cache should be invalidated or use some other 
   * algorithm to find out all the children of that node and remove from the cache
   */
  final public void changeNode(String uri) throws Exception {
    NodeDescriptor uiNode = (NodeDescriptor) cacheNodes_.get(uri) ;
    if (uiNode == null) { 
      synchronized (cacheNodes_) {
        uiNode = (NodeDescriptor)cacheNodes_.get(uri) ;
        if (uiNode == null) {
          String parentUri = getParentUri(uri) ;
          String realPath = baseDir_ + uri ;
          File file = new File(realPath) ;
          uiNode = new NodeDescriptor(file, uri, parentUri) ;
          cacheNodes_.put(uri, uiNode) ;
          File[] children = file.listFiles() ;
          List nodeChildren = uiNode.getChildren() ;
          parentUri = uri ;
          if ("/".equals(parentUri)) parentUri = "" ;
          for (int i = 0 ; i < children.length ; i++) {
            File child = children[i] ;
            String childUri = parentUri + "/"  + child.getName() ;
            NodeDescriptor uiChild = new NodeDescriptor(child, childUri, uri) ;
            nodeChildren.add(uiChild) ;
          }
        }
      }
    }
    setSelectNode(uiNode) ;
  }
  
  private  void  removeNode(String uri) throws Exception {
    String realPath = baseDir_ + uri ;
    File file = new File(realPath) ;
    boolean isDir = file.isDirectory() ;
    file.delete() ;
    String parentUri = getParentUri(uri) ;
    if (isDir) {
      synchronized (cacheNodes_) {
        cacheNodes_.clear() ;
      }
    } else {
      synchronized (cacheNodes_) {
        cacheNodes_.remove(parentUri) ;
      }
    }
    //update the select node
    changeNode(parentUri) ;
  }
	
  public void updateCache(String uri) throws Exception {
    String realPath = baseDir_ + uri ;
    File file = new File(realPath) ;
    String parentUri = getParentUri(uri) ;
    boolean isDir = file.isDirectory() ;
    if (isDir) {
      synchronized (cacheNodes_) {
        cacheNodes_.clear() ;
      }
    } else {
      synchronized (cacheNodes_) {
        cacheNodes_.remove(parentUri) ;
      }
    }
    //update the select node
    changeNode(parentUri) ;
  }
  
	static public class ChangeNodeActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIFileExplorer uiExplorer = (UIFileExplorer) event.getComponent() ;
			String uri = event.getParameter("uri") ;
 		  if("../".equals(uri))  uri = uiExplorer.getSelectNode().getParentUri() ;
			uiExplorer.changeNode(uri);
		}
	}
	
	static public class RemoveNodeActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIFileExplorer uiExplorer = (UIFileExplorer) event.getComponent() ;
			InformationProvider iprovider = findInformationProvider(uiExplorer)  ;
      if(!uiExplorer.acl_.hasWriteRole(null)) {
       iprovider.addMessage(new ExoFacesMessage("#{UIFileExplorer.error.write-permission}")) ;
      } else {
      	String uri = event.getParameter("uri") ;
      	if("../".equals(uri))  uri = uiExplorer.getSelectNode().getParentUri() ;
      	uiExplorer.removeNode(uri) ;
      	iprovider.addMessage(new ExoFacesMessage("#{UIFileExplorer.msg.delete-file-success}")) ;
      }
		}
	}
	
	static public class EditContentActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIFileExplorer uiExplorer = (UIFileExplorer) event.getComponent() ;
			if(!uiExplorer.acl_.hasWriteRole(uiExplorer.getSelectNode())) {
				InformationProvider iprovider = findInformationProvider(uiExplorer) ;
			  iprovider.addMessage(new ExoFacesMessage("#{UIFileExplorer.error.write-permission}")) ;
				return ;
			}
			UIFilePortlet uiController = (UIFilePortlet) uiExplorer.getParent() ;
			UIFileContentEditor uiEditor = 
      	(UIFileContentEditor)uiController.getChildComponentOfType(UIContentEditor.class) ;
			
			String parentUri = event.getParameter("parentUri") ;
		  String localName = event.getParameter("localName") ;
		  String content = "" ;
		  String mimeType = "text/plain" ;
		  if (localName != null) {
		  	mimeType = MimeTypeUtil.getMimeType(localName) ;
		    if (!mimeType.startsWith("text")) {
		    	InformationProvider iprovider = findInformationProvider(uiExplorer) ;
		    	iprovider.addMessage(new ExoFacesMessage("mime type " + mimeType + " is not supported")) ;
		    	return ;
		    }
		    String path = uiExplorer.baseDir_ + parentUri + "/" + localName ;
		    FileInputStream is = new FileInputStream(path) ;
		    byte[] buf = new byte[is.available()] ;
		    is.read(buf) ;
		    is.close() ;
		    content =  new String(buf);
		  }
		  uiEditor.populateEdit(parentUri, localName, mimeType, content) ;
		  uiController.setRenderedComponent(uiEditor.getId()) ;
		}
	}
	
	static public class ViewContentActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIFileExplorer uiExplorer = (UIFileExplorer) event.getComponent() ;
			UIFilePortlet uiController = (UIFilePortlet) uiExplorer.getParent() ;
			UIContentDisplayer uiDisplayer = 
				(UIContentDisplayer) uiController.getChildComponentOfType(UIContentDisplayer.class) ;
			String uri = event.getParameter("uri") ;
			String realPath = uiExplorer.baseDir_ + uri ;
	    File file = new File(realPath) ;
	    String mimeType = MimeTypeUtil.getMimeType(file.getName()) ;
	    Renderer renderer =   uiDisplayer.getRenderer(mimeType) ;
	    String content = "" ;
	    if (renderer != null) {
	      if (mimeType.startsWith("text")) {
	        FileInputStream is = new FileInputStream(file) ;
	        byte[] buf = new byte[is.available()] ;
	        is.read(buf) ;
	        is.close() ;
	        content =  new String(buf);
	      } else {
	        content =  "/content/FileContentServlet?path=" + realPath ;
	      }
	    } else {
	      renderer = uiDisplayer.getRenderer("default") ;
	      System.out.println("renderer in else = " + renderer);
	      content =  "/content/FileContentServlet?path=" + realPath ;
	    }
	    uiDisplayer.populate(uri, mimeType, content, renderer) ;
	    uiController.setRenderedComponent(uiDisplayer.getId()) ;
		}
	}
	
	static public class UploadFileActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIFileExplorer uiExplorer = (UIFileExplorer) event.getComponent() ;
			if(!uiExplorer.acl_.hasWriteRole(uiExplorer.getSelectNode())) {
				InformationProvider iprovider = findInformationProvider(uiExplorer) ;
			  iprovider.addMessage(new ExoFacesMessage("#{UIFileExplorer.error.write-permission}")) ;
				return ;
			}
			UIPortlet uiParent = (UIPortlet) uiExplorer.getParent() ;
			UIFileUpload uiFileUpload = 
				(UIFileUpload)uiParent.getChildComponentOfType(UIFileUpload.class) ;
			uiFileUpload.setParentUri(uiExplorer.getSelectNode().getUri()) ;
			uiParent.setRenderedComponent(uiFileUpload.getId()) ;
		}
	}
}