/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.explorer.component;

import java.io.FileOutputStream;
import java.util.Iterator;
import java.util.List;
import javax.faces.context.FacesContext;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIFileUpload;
import org.exoplatform.faces.core.component.UIPortlet;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.ACL;
import org.exoplatform.portlets.content.FileACL;

/**
 * Wed, Dec 22, 2003 @ 23:14 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIFilePortlet.java,v 1.3 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UIFilePortlet extends UIPortlet {
  private ACL acl_ ;
  private String baseDir_ ;
  
  public UIFilePortlet(UIFileExplorer uiExplorer, UIFileContentEditor uiEditor,
			                 UIContentDisplayer uiDisplayer, UIFileUpload uiUpload) throws Exception {
  	FacesContext context = FacesContext.getCurrentInstance() ;
    PortletRequest request = (PortletRequest) context.getExternalContext().getRequest() ;
    PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
    String user = request.getRemoteUser() ;
    if(user == null) user = "" ;
    String readRole = prefs.getValue("read-role", "any") ;
    String writeRole = prefs.getValue("write-role", "admin") ;
    baseDir_ = prefs.getValue("base-dir", "default") ;
    if(baseDir_.equals("default")) {
    	baseDir_ =  FacesUtil.getApplicationRealPath("/") ;
    }
    acl_ = new FileACL(user, readRole, writeRole) ;
  	setRendererType("FilePortletRenderer") ;
  	
  	List children = getChildren() ;
  	uiExplorer.setRendered(true) ;
  	uiExplorer.init(acl_ , baseDir_ ) ;
  	children.add(uiExplorer) ;

  	uiDisplayer.setRendered(false) ;
  	children.add(uiDisplayer) ;

  	uiEditor.setRendered(false) ;
  	children.add(uiEditor) ;

  	uiUpload.setRendered(false) ;
  	uiUpload.addActionListener(ShowExplorerActionListener.class, UIFileUpload.CANCEL_ACTION) ;
  	uiUpload.addActionListener(SaveUploadFileActionListener.class, UIFileUpload.UPLOAD_ACTION) ;
  	children.add(uiUpload) ;
  }
  
  public String getBaseDir() { return baseDir_ ; }
  
  public String getFamily() {
  	return "org.exoplatform.portlets.content.component.UIFilePortlet" ;
  }
  
  static public class SaveUploadFileActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIFileUpload uiFileUpload = (UIFileUpload) event.getComponent() ;
			UIFilePortlet uiController = (UIFilePortlet) uiFileUpload.getParent() ;
			InformationProvider iprovider = findInformationProvider(uiFileUpload)  ;
			if(!uiController.acl_.hasWriteRole(null)) {
        iprovider.addMessage(new ExoFacesMessage("#{UIFilePortlet.error.write-permission}")) ;
      }  else {
        String dir = uiController.baseDir_ + uiFileUpload.getParentUri() ;
        Iterator i = uiFileUpload.getUserInputs().iterator() ;
        while(i.hasNext()) {
          UIFileUpload.UserInput input = (UIFileUpload.UserInput) i.next() ;
          String fileName = input.getName() ;
          if (fileName == null || fileName.length() == 0) fileName = input.getFileName() ;
          String realPath = dir + "/" + fileName ;
          FileOutputStream os = new FileOutputStream(realPath) ;
          os.write(input.getStream()) ;
          os.close() ;
        }
        UIFileExplorer uiExplorer = 
        	(UIFileExplorer) uiController.getChildComponentOfType(UIFileExplorer.class) ;
        uiExplorer.updateCache(uiFileUpload.getParentUri()) ;
        iprovider.addMessage(new ExoFacesMessage("#{UIFilePortlet.msg.file-upload-success}")) ;
      }
		}
	}
}