/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.explorer.renderer.html;

import java.io.IOException;
import java.util.List;
import java.util.ResourceBundle;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import org.apache.commons.lang.StringUtils;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.renderer.html.HtmlBasicRenderer;
import org.exoplatform.portlets.content.explorer.component.UIExplorer;
import org.exoplatform.portlets.content.explorer.component.model.NodeDescriptor;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: ExplorerRenderer.java,v 1.3 2004/09/05 22:06:15 tuan08 Exp $
 */
public class ExplorerRenderer extends HtmlBasicRenderer {
	final public static String VIEW_ICON = "<img class='view-icon' src='/skin/blank.gif'/>";
	final public static String EDIT_ICON = "<img class='edit-icon' src='/skin/blank.gif'/>";
	final public static String DELETE_ICON = "<img class='delete-icon' src='/skin/blank.gif'/>";

  private static Parameter SHOW_EDITOR = 	new Parameter(ACTION , UIExplorer.EDIT_NODE_CONTENT) ;
  private static Parameter UPLOAD_FILE = new Parameter(ACTION , UIExplorer.UPLOAD_FILE) ;
  private static Parameter NEW_FOLDER = new Parameter(ACTION , "newFolder") ;
  private static Parameter CHANGE_NODE = new Parameter(ACTION , UIExplorer.CHANGE_NODE) ;
  
  private static Parameter DELETE =  new Parameter(ACTION , UIExplorer.REMOVE_NODE) ;
  private static Parameter VIEW =  new Parameter(ACTION , UIExplorer.VIEW_NODE_CONTENT) ;
  private static Parameter EDIT =  new Parameter(ACTION , UIExplorer.EDIT_NODE_CONTENT) ;

  public void encodeChildren( FacesContext context, UIComponent component ) throws IOException {
    UIExplorer uiExplorer = (UIExplorer) component;
    ResourceBundle res = getApplicationResourceBundle(context.getExternalContext()) ;
    String currentUri = uiExplorer.getSelectNode().getUri() ;
    String[] path = StringUtils.split(currentUri, "/") ; 
    if(path.length == 0)  {
      path = new String[0] ; 
    }
    String currentPath =   ""  ; 
    Parameter uriParam = new Parameter("uri", "/") ;
    Parameter[] changeNodeParams = { CHANGE_NODE , uriParam} ;
    ResponseWriter w = context.getResponseWriter() ;
    //render header
    w.write("<table class='UIExplorer'>");
    w.  write("<tr>");
    w.    write("<td class='header' colspan='2'>") ;
    linkRenderer_.render(w, uiExplorer,  "/",  changeNodeParams) ;
    String pathSeparator = res.getString("UIExplorer.img.path-separator") ;
    for (int i = 0; i < path.length; i++) {
      currentPath += "/" +  path[i] ; 
      uriParam.setValue(currentPath) ;
      w.write("<img src='"); w.write(pathSeparator) ; w.write("'/>") ;
      uiExplorer.appendLink(w, path[i], changeNodeParams, "") ;
    }
    w.    write("</td>") ;
    w.  write("</tr>");
  	//render body
    w.  write("<tr>");
    w.    write("<td>") ;
    renderNodeTree(w, res, uiExplorer) ;
    w.    write("</td>");
    w.    write("<td>") ;
    renderNodeDetail(w, res, uiExplorer) ;
    w.    write("</td>");
    w. write("</tr>");
    //render footer 
    NodeDescriptor uiNodeDescriptor =  uiExplorer.getSelectNode() ;
    Parameter parentUriParam = new Parameter("parentUri", uiNodeDescriptor.getUri()) ;
    Parameter[] showEditorParams = { SHOW_EDITOR , parentUriParam } ;
    Parameter[] showUploadParams = { UPLOAD_FILE , parentUriParam } ;
    Parameter[] newFolderParams = { NEW_FOLDER , parentUriParam } ;
    w.  write("<tr>");
    w.    write("<td class='footer' colspan='2'>") ;
    uiExplorer.appendLink(w, "Editor", showEditorParams, "") ;
    uiExplorer.appendLink(w, "Upload", showUploadParams, "") ;
    uiExplorer.appendLink(w, "New Folder", newFolderParams, "") ;
    w.    write("</td>");
    w. write("</tr>");
    w.write("</table>");
  }
  
  final public void  renderNodeTree(ResponseWriter w, ResourceBundle res, 
  		                              UIExplorer uiExplorer) throws IOException {
  	NodeDescriptor node = uiExplorer.getSelectNode() ;
    String closeFolderImg =  res.getString("UIExplorer.img.close-folder") ;
    Parameter uriParam = new Parameter("uri", "../") ;
    Parameter[] changeNodeParams = { CHANGE_NODE , uriParam} ;
    w.write("<div class='tree'>") ;
    uiExplorer.appendLink(w, "[..]", changeNodeParams, "") ;
    w.write("</div>") ;
    List children = node.getChildren() ;
    for (int i = 0; i < children.size(); i++) {
      NodeDescriptor childNode = (NodeDescriptor) children.get(i) ;
      if (!childNode.isLeafNode()) {
        uriParam.setValue(childNode.getUri()) ;
        w.write("<div style='padding: 2px'>") ;
        w.write("<img src='"); w.write(closeFolderImg); w.write("'/>") ;
        uiExplorer.appendLink(w, childNode.getName(), changeNodeParams, "") ;
        w.write("</div>") ;
      }
    }
  }
  
  private void renderNodeDetail(ResponseWriter w, ResourceBundle res, 
                                UIExplorer uiExplorer) throws IOException {
  	NodeDescriptor node = uiExplorer.getSelectNode() ;
    Parameter uriParam = new Parameter("uri", "") ;
    Parameter parentUriParam = new Parameter("parentUri", "") ;
    Parameter localNameParam = new Parameter("localName", "") ;
    Parameter[] deleteNodeParams = { DELETE , uriParam} ;
    Parameter[] editNodeParams = { EDIT , parentUriParam, localNameParam} ;
    Parameter[] viewNodeParams = { VIEW , uriParam} ;
    List children = node.getChildren() ;
    //write the header
    w.write("<table class='detail'>") ;
    w.  write("<tr>");
    w.    write("<th>") ; w.write(res.getString("UIExplorer.header.locale-name")) ; w.write("</th>") ;
    w.    write("<th>") ; w.write(res.getString("UIExplorer.header.modified-date")) ; w.write("</th>") ;
    w.    write("<th>") ; w.write(res.getString("UIExplorer.header.created-date")) ; w.write("</th>") ;
    w.    write("<th>") ; w.write(res.getString("UIExplorer.header.owner")) ; w.write("</th>") ;
    w.    write("<th>") ; w.write(res.getString("UIExplorer.header.versioning")) ; w.write("</th>") ;
    w.    write("<th>") ; w.write(res.getString("UIExplorer.header.action")) ; w.write("</th>") ;
    w.  write("</tr>") ;
    //write the children rows
    for(int i = 0; i < children.size(); i++) {
      NodeDescriptor uiChild = (NodeDescriptor) children.get(i) ;
      String versioning = "false" ;
      if (uiChild.isVersioning()) versioning = "true" ;
      uriParam.setValue(uiChild.getUri()) ;
      parentUriParam.setValue(uiChild.getParentUri()) ;
      localNameParam.setValue(uiChild.getName()) ;

      String clazz = "odd";
      if (i % 2 == 0) clazz = "even";
      w.write("<tr class='" + clazz + "'>") ;
      w.  write("<td style='text-align: left'>") ; w.write(uiChild.getName()) ; w.write("</td>") ;
      w.  write("<td>") ; w.write(ft_.format(uiChild.getModifiedDate())) ; w.write("</td>") ;
      w.  write("<td>") ; w.write(ft_.format(uiChild.getCreatedDate())) ; w.write("</td>") ;
      w.  write("<td>") ; w.write(ft_.format(uiChild.getOwner())) ; w.write("</td>") ;
      w.  write("<td>") ; w.write(versioning) ; w.write("</td>") ;
      w.  write("<td>") ;
      if (uiChild.isLeafNode()) {
        linkRenderer_.render(w,uiExplorer, VIEW_ICON, viewNodeParams) ;
        linkRenderer_.render(w,uiExplorer, EDIT_ICON, editNodeParams) ;
      } else {
      }
      uiExplorer.appendLink(w, DELETE_ICON, deleteNodeParams, "") ;
      w.  write("</td>") ;
      w.write("</tr>") ;
    }
    w.write("</table>");
  }
}