package org.exoplatform.portlets.nav.component;

import java.util.ResourceBundle;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.ExoPortalInfo;
import org.exoplatform.services.portal.model.Node;
/**
 * @version $Revision: 1.11 $ $Date: 2004/09/27 18:26:58 $
 * @author Fahrid Djebbari
 */
public class UINavigation extends UIExoCommand {
  final public static Parameter CHANGE_NODE = new Parameter(ACTION, "changeNode");
  final static public Parameter CHANGE_MODE =   new Parameter(ACTION, "changeMode");
  final static public Parameter[] viewModeParams = {CHANGE_MODE,  new Parameter("mode", "normal") };
  final static public Parameter[] editNavigationModeParams = 
    {CHANGE_MODE,  new Parameter("mode", "edit-navigation") };
  final static public Parameter[] editPortalModeParams =  {CHANGE_MODE,  new Parameter("mode", "edit-portal") };
  final static public Parameter[] editPageModeParams =  {CHANGE_MODE,  new Parameter("mode", "edit-page") };
  
	final static public String DEFAULT_RENDERER_TYPE = "HorizontalMenuRenderer" ;
  
  protected Node rootNode_ ;
  protected ExoPortalInfo portalInfo_ ;
  private boolean adminRole_ = false ;
  private boolean showReturn_  = false ;
  
	public UINavigation(ResourceBundle res) throws Exception {
		setId("UINavigation");
		setRendererType(DEFAULT_RENDERER_TYPE);
		setTransient(false);
    portalInfo_ = ExoPortalInfo.getCurrentInstance() ;
    rootNode_ = portalInfo_.getRootNode() ;
    String contextPath = portalInfo_.getAccessibility() ;
    if(ExoPortalInfo.PRIVATE_PATH.equals(contextPath)) {
    	adminRole_ = true ;
    } else if (ExoPortalInfo.ADMIN_PATH.equals(contextPath)) {
      adminRole_ = true ;
      showReturn_ = true ;
    } else {
    	adminRole_  = false ;
    }
    
    addActionListener(ChangeNodeActionListener.class, "changeNode") ;
    addActionListener(ChangeModeActionListener.class, "changeMode") ;
	}
	
  public ExoPortalInfo getPortalInfo() { return portalInfo_ ; }
  
  public Node getRootNode() { return rootNode_ ; }
  public Node getSelectedNode()  {  return  portalInfo_.getSelectedNode() ; }
  
  public String getPreferedMimeType() { return portalInfo_.getPreferredMimeType() ; }

  public boolean hasAdminRole() { return adminRole_ ; }
  
  public boolean getShowReturn() { return showReturn_ ; }
  
  public void setViewModeRenderer() {  }
  
  public void setEditPortalModeRenderer() {  }
  
  public void setEditPageModeRenderer() {  }
  
  public void setEditNavigationModeRenderer() {  }
  
  public String getFamily( ) { return "org.exoplatform.portlets.nav.component.UINavigation" ; }
  
	static public class ChangeNodeActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
      UINavigation uiNavigation = (UINavigation) event.getSource() ;
			String uri = event.getParameter("uri");
			Node node = uiNavigation.rootNode_.findNode(uri) ;
			uiNavigation.portalInfo_.setSelectedNode(node) ;
		}
	}
	
  static public class ChangeModeActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
      UINavigation uiNavigation = (UINavigation) event.getSource() ;
			String mode = event.getParameter("mode");
			if("edit-navigation".equals(mode)) {
				uiNavigation.portalInfo_.setPortalMode(ExoPortalInfo.EDIT_NAVIGATION_MODE) ;
        uiNavigation.setEditNavigationModeRenderer() ;
			} else if("edit-portal".equals(mode)) {
				uiNavigation.portalInfo_.setPortalMode(ExoPortalInfo.EDIT_PORTAL_MODE) ;
        uiNavigation.setEditPortalModeRenderer() ;
			} else if("edit-page".equals(mode)) {
				uiNavigation.portalInfo_.setPortalMode(ExoPortalInfo.EDIT_PAGE_MODE) ;
        uiNavigation.setEditPageModeRenderer() ;
			} else {
				uiNavigation.portalInfo_.setPortalMode(ExoPortalInfo.VIEW_MODE) ;
        uiNavigation.setViewModeRenderer() ;
			}
		}
	}
}