/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;
import org.exoplatform.faces.FacesConstants;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIFormWithInformationProvider;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.EmailAddressValidator;
import org.exoplatform.faces.core.validator.EmptyStringValidator;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIAccountForm.java,v 1.14 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UIAccountForm extends UIFormWithInformationProvider {
  private User user_ ;
  private UIStringInput userNameInput_ ;
  private UIStringInput password1xInput_ ;
  private UIStringInput password2xInput_ ;
  private UIStringInput firstNameInput_ ;
  private UIStringInput lastNameInput_ ;
  private UIStringInput emailInput_ ;
  private FormButton saveButton_ ;
  private OrganizationService service_ ;
  private FormButton cancelButton_;
  
  public UIAccountForm(OrganizationService service) throws Exception {
    super("accountForm", "post", null) ;
    setId("UIAccountForm");
    service_ = service ;
    init() ;
  }
   
  private void init() {
    userNameInput_ = new UIStringInput("userName", "").
                     addValidator(EmptyStringValidator.class) ;
    password1xInput_ = new UIStringInput("password1x", "").
                       addValidator(EmptyStringValidator.class) ;
    password1xInput_.setType(UIStringInput.PASSWORD);
    password2xInput_ = new UIStringInput("password2x", "").
                       addValidator(EmptyStringValidator.class) ;
    password2xInput_.setType(UIStringInput.PASSWORD);
    firstNameInput_ = new UIStringInput("firstName", "") ;
    lastNameInput_ = new UIStringInput("lastName", "") ;
    emailInput_ = new UIStringInput("email", "").
                  addValidator(EmailAddressValidator.class) ;
    saveButton_ = new FormButton("#{UIAccountForm.button.save}", SAVE_ACTION) ;
    cancelButton_ = new FormButton("#{UIAccountForm.button.cancel}", CANCEL_ACTION) ;
    cancelButton_.setVisible(false);

    add(new HeaderRow().
        add(new Cell("#{UIAccountForm.header.new-account}").
            addColspan("2")));
    add(new Row().
        add(new LabelCell("#{UIAccountForm.label.user-name}")).
        add(new ComponentCell(this, userNameInput_)));
    add(new Row().
        add(new LabelCell("#{UIAccountForm.label.password1x}")).
        add(new ComponentCell(this, password1xInput_)));
    add(new Row().
        add(new LabelCell("#{UIAccountForm.label.password2x}")).
        add(new ComponentCell(this, password2xInput_)));
    add(new Row().
        add(new LabelCell("#{UIAccountForm.label.first-name}")).
        add(new ComponentCell(this, firstNameInput_)));
    add(new Row().
        add(new LabelCell("#{UIAccountForm.label.last-name}")).
        add(new ComponentCell(this, lastNameInput_)));
    add(new Row().
        add(new LabelCell("#{UIAccountForm.label.email}")).
        add(new ComponentCell(this, emailInput_)));
    add(new Row().
        add(new ListComponentCell().
            add(saveButton_).
            add(cancelButton_).
            addColspan("2").addAlign("center"))) ;
    addValidator(ConfirmPasswordValidator.class);
    addActionListener(NewAccountListener.class, SAVE_ACTION) ;
    addActionListener(CancelListener.class, CANCEL_ACTION) ;
    addActionListener(UpdateAccountListener.class, FacesConstants.UPDATE_ACTION) ;
  }
  
  public void setEditingUser(String userName) throws Exception {
    user_ = service_.findUserByName(userName) ;
    userNameInput_.setValue(user_.getUserName());
    userNameInput_.setEditable(false) ;
    password1xInput_.setValue(user_.getPassword());
    password2xInput_.setValue(user_.getPassword());
    firstNameInput_.setValue(user_.getFirstName());
    lastNameInput_.setValue(user_.getLastName());
    emailInput_.setValue(user_.getEmail());
    saveButton_.setAction(FacesConstants.UPDATE_ACTION) ;
    cancelButton_.setVisible(true);
  }
  
  static public class ConfirmPasswordValidator implements Validator {
    public void validate(FacesContext context, UIComponent component,
                         Object value) throws ValidatorException {
      UIAccountForm uiForm = (UIAccountForm) component ;
      if(!uiForm.password1xInput_.getValue().equals(uiForm.password2xInput_.getValue())) {
        throw new ValidatorException(new ExoFacesMessage("#{UIAccountForm.msg.password-error}")) ;
      }
    }
  }
  
  static public class NewAccountListener extends ExoActionListener  {
  	public void execute(ExoActionEvent event) throws Exception {
  	  UIAccountForm uicomp = (UIAccountForm) event.getComponent() ;    
  	  InformationProvider iprovider = findInformationProvider(uicomp);
  		String userName = uicomp.userNameInput_.getValue() ;
      String password1x = uicomp.password1xInput_.getValue() ;
      String firstName = uicomp.firstNameInput_.getValue() ;
      String lastName = uicomp.lastNameInput_.getValue()  ;
      String email = uicomp.emailInput_.getValue() ;
      User newUser = uicomp.service_.findUserByName(userName) ;
      if (newUser == null) {
        newUser = uicomp.service_.createUserInstance() ;
        newUser.setUserName(userName) ;
        newUser.setPassword(password1x) ;
        newUser.setFirstName(firstName) ;
        newUser.setLastName(lastName) ;
        newUser.setEmail(email) ;
        uicomp.service_.createUser(newUser);
       
        uicomp.userNameInput_.setValue("");
        uicomp.password1xInput_.setValue("");
        uicomp.password2xInput_.setValue("");
        uicomp.firstNameInput_.setValue("");
        uicomp.lastNameInput_.setValue("");
        uicomp.emailInput_.setValue("");
        iprovider.addMessage(new ExoFacesMessage("#{UIAccountForm.msg.create-user-success}")) ;
      } else {
        iprovider.addMessage(new ExoFacesMessage("#{UIAccountForm.msg.user-already-exist}")) ;
      }
    }
  }
  
  static public class CancelListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIAccountForm uicomp = (UIAccountForm) event.getComponent() ;    
      uicomp.setRenderedSibling(UIUserInfo.class) ;
    }
  }  
  
  static public class UpdateAccountListener extends ExoActionListener  {
  	public void execute(ExoActionEvent event) throws Exception {
  	  UIAccountForm  uicomp = (UIAccountForm) event.getComponent() ;
  	  String password1x = uicomp.password1xInput_.getValue() ;
      String firstName =  uicomp.firstNameInput_.getValue() ;
      String lastName =   uicomp.lastNameInput_.getValue()  ;
      String email = uicomp.emailInput_.getValue() ;
      uicomp.user_.setPassword(password1x) ;
      uicomp.user_.setFirstName(firstName) ;
      uicomp.user_.setLastName(lastName) ;
      uicomp.user_.setEmail(email) ;
      uicomp.service_.saveUser(uicomp.user_) ;
      UIUserInfo uiUserInfo = 
      	(UIUserInfo)uicomp.getSibling(UIUserInfo.class) ;
      uiUserInfo.update() ;
      uicomp.setRenderedSibling(UIUserInfo.class) ;
    }
  }
}