/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import org.exoplatform.commons.exception.ExoMessageException;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIMembershipForm.java,v 1.11 2004/10/22 14:23:30 tuan08 Exp $
 */
public class UIMembershipForm extends UISimpleForm {
  private UIStringInput userNameInput_ ;
  private UISelectBox   membershipNameInput_ ;
  private UIStringInput groupIdInput_ ;
  private OrganizationService service_ ;

  public UIMembershipForm(OrganizationService service) throws Exception {
    super("membershipForm", "post", null) ;
    setId("UIMembershipForm");
    userNameInput_ = new UIStringInput("userName", "") ;
    membershipNameInput_ = new UISelectBox("roleName", "", null) ;    
    groupIdInput_ = new UIStringInput("groupId", "") ;
    service_ = service ;
    add(new HeaderRow().
        add(new Cell("#{UIMembershipForm.header.membership}").
            addColspan("2")));
    add(new Row().
        add(new LabelCell("#{UIMembershipForm.label.user-name}")).
        add(new ComponentCell(this, userNameInput_)));
    add(new Row().
        add(new LabelCell("#{UIMembershipForm.label.membership-name}")).
        add(new ComponentCell(this, membershipNameInput_)));
    add(new Row().
        add(new LabelCell("#{UIMembershipForm.label.group-id}")).
        add(new ComponentCell(this, groupIdInput_)));
    add(new Row().
        add(new ListComponentCell().
            add(new FormButton("#{UIMembershipForm.button.save}", SAVE_ACTION)).
            add(new FormButton("#{UIMembershipForm.button.cancel}", CANCEL_ACTION)).
            addColspan("2").addAlign("center"))) ;
    
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
  }
 
  public void addMembership(String userName) throws Exception {
    Collection memberships = service_.findMembershipTypes() ;
    Iterator i = memberships.iterator() ;
    List roleOptions = new ArrayList(10) ;
    while(i.hasNext()) {
      MembershipType mt = (MembershipType) i.next();
      String name = mt.getName() ;
      roleOptions.add(new SelectItem(name, name)) ;
    }

    userNameInput_.setValue(userName) ;
    userNameInput_.setEditable(false) ;
    membershipNameInput_.setValue("") ;
    membershipNameInput_.setOptions(roleOptions) ;    
    groupIdInput_.setValue("") ;
  }
  
  static public class SaveActionListener extends ExoActionListener  {
    public SaveActionListener( ) {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
  	public void execute(ExoActionEvent event) throws Exception {
      UIMembershipForm uiForm = (UIMembershipForm) event.getSource() ;
  		String userName = uiForm.userNameInput_.getValue() ;
      String groupId = uiForm.groupIdInput_.getValue();
      Group group = uiForm.service_.findGroupById(groupId) ;
      if(group == null) {
        Object[] args = { groupId } ;
        throw new ExoMessageException("UIMembershipForm.group-not-found", args) ;
      }
  		Membership membership = uiForm.service_.createMembershipInstance();
      
      membership.setMembershipType(uiForm.membershipNameInput_.getValue());
      uiForm.service_.linkMembership(userName, group, membership);
      UIUserInfo uiUserInfo = 
      	(UIUserInfo) uiForm.getSibling(UIUserInfo.class) ;
      uiUserInfo.update();
      uiForm.setRenderedSibling(UIUserInfo.class) ;
    }
  }
  
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIMembershipForm uiForm = (UIMembershipForm) event.getSource() ;
      uiForm.setRenderedSibling(UIUserInfo.class) ;
    }
  }
}