/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.Map;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.portlet.PortletRequest;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.event.CheckAdminOrOwnerRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.ExoPortalInfo;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.UserProfile;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIUserProfile.java,v 1.14 2004/09/28 18:19:23 tuan08 Exp $
 */
public class UIUserProfile extends UIExoCommand {
	final private static String VIEW_RENDERER = "UserProfileRenderer" ;
	final private static String FORM_RENDERER = "UserProfileFormRenderer" ;
  final static public String[] PERSONAL_INFO_KEYS =
   {
      "user.bdate" , "user.gender" , "user.employer" , "user.department", 
      "user.jobtitle", "user.name.given", "user.name.family", "user.name.nickName"
   } ;

  final static public String[] HOME_INFO_KEYS =
   {
     "user.home-info.postal.name", "user.home-info.postal.street", 
     "user.home-info.postal.city", "user.home-info.postal.stateprov" ,
     "user.home-info.postal.postalcode", "user.home-info.postal.country",
     "user.home-info.telecom.mobile.number", "user.home-info.telecom.telephone.number", 
     "user.home-info.online.email", "user.home-info.online.uri" 
   } ;

  final static public String[] BUSINESE_INFO_KEYS =
   {
     "user.business-info.postal.name", "user.business-info.postal.city", 
     "user.business-info.postal.stateprov", "user.business-info.postal.postalcode",
     "user.business-info.postal.country", "user.business-info.telecom.mobile.number",
     "user.business-info.telecom.telephone.number", "user.business-info.online.email",
     "user.business-info.online.uri"
   } ;
  
  private String userName_ ;
  private Map userInfoMap_ ;
  private boolean editable_ ;
  private boolean adminRole_ ;
  private boolean showBackButton_ ;
  private OrganizationService service_ ;
   
  public UIUserProfile(OrganizationService service) throws Exception {
  	setId("UIUserProfile") ;
  	setRendererType(VIEW_RENDERER) ;
    service_ = service ;
    ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext() ;
    Map requestMap = eContext.getRequestMap() ;
    ExoPortalInfo pinfo = ExoPortalInfo.getCurrentInstance() ;
    userInfoMap_ = (Map) requestMap.get(PortletRequest.USER_INFO) ;
    userName_ = pinfo.getPortalOwner();
    editable_ = hasOwnerRole(userName_) ;
    adminRole_ = hasRole("admin") ;
    showBackButton_ = false  ;
    addActionListener(BackActionListener.class , BACK_ACTION) ;
    addActionListener(CancelActionListener.class , CANCEL_ACTION) ;
    addActionListener(EditActionListener.class , EDIT_ACTION) ;
    addActionListener(SaveActionListener.class , SAVE_ACTION) ;
  }
 
  
  public boolean isEditable() { return editable_ ; } 
  public void    setEditable(boolean b) { editable_ = b ; }
  
  public boolean showBackButton() { return showBackButton_ ; }
  
  public Map getUserInfoMap() { return userInfoMap_ ; } 
  
  public void setUserProfile(String userName) throws Exception { 
  	userName_ = userName ;
    userInfoMap_ = service_.findUserProfileByName(userName).getUserInfoMap() ;
    editable_ = adminRole_ ;
    showBackButton_ = true  ;
  }
  
  private void save(Map paramMap) throws Exception {
  	update(userInfoMap_, paramMap, PERSONAL_INFO_KEYS) ;
    update(userInfoMap_, paramMap, HOME_INFO_KEYS) ;
    update(userInfoMap_, paramMap, BUSINESE_INFO_KEYS) ;
    UserProfile profile = service_.findUserProfileByName(userName_) ;
    profile.setUserInfoMap(userInfoMap_) ;
    service_.saveUserProfile(profile) ;
  }
  
  private void update(Map userInfo, Map paramMap, String[] keys) {
    for (int i = 0 ; i < keys.length; i++) {
      String value = (String) paramMap.get(keys[i]) ;
      if (value == null  || value.length() == 0) {
        userInfo.remove(keys[i]) ;
      } else {
        userInfo.put(keys[i], value) ;
      }
    }
  }
  
  static public class BackActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIUserProfile uiProfile = (UIUserProfile) event.getComponent() ;
      uiProfile.setRenderedSibling(UIListUser.class) ;
    }
  }
  
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIUserProfile uiProfile = (UIUserProfile) event.getComponent() ;
      uiProfile.setRendererType(VIEW_RENDERER) ;
    }
  }
  
  static public class EditActionListener extends ExoActionListener  {
    public EditActionListener() {
      addInterceptor(new CheckAdminOrOwnerRoleInterceptor()) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIUserProfile uiProfile = (UIUserProfile) event.getComponent() ;
      uiProfile.setRendererType(FORM_RENDERER) ;
    }
  }
  
  static public class SaveActionListener extends ExoActionListener  {
    public SaveActionListener() {
      addInterceptor(new CheckAdminOrOwnerRoleInterceptor()) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIUserProfile uiProfile = (UIUserProfile) event.getComponent() ;
      Map paramMap = event.getParameterMap() ;
      uiProfile.save(paramMap) ;
    }
  }
}