package org.exoplatform.services.log.impl;

import java.io.PrintStream;
import java.util.ArrayList ;
import java.util.List ;
import org.apache.commons.logging.Log;
import org.exoplatform.commons.utils.ExceptionUtil;
import org.exoplatform.services.log.*;


/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

/**
 * Created by the Exo Development team.
 * Author : Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 * Date: 14 nov. 2003
 * Time: 21:47:20
 */
public class ExoLog implements Log {
  static public int FATAL = 0 ;
  static public int ERROR = 1 ;
  static public int WARN = 2 ;
  static public int INFO = 3 ;
  static public int DEBUG = 4 ;
  static public int TRACE = 5 ;

  static public int LOG_BUFFER_SIZE = 2000 ;
  static public int ERROR_BUFFER_SIZE = 1500 ;

  static private List logBuffer_ = new ArrayList(LOG_BUFFER_SIZE * 2) ;
  static private List errorBuffer_ = new ArrayList(ERROR_BUFFER_SIZE * 2) ;
  
  private String category_ ;
  private int level_ ;
  private PrintStream out_ ;
  private PrintStream err_ ;
  private String prefix_ ;

  public ExoLog(String name, int level, PrintStream out, PrintStream err) {
    category_ = name ;
    level_ = level ;
    out_ =  out;
    err_ =  err ;
    int index = name.lastIndexOf(".") ;
    String nameSuffix = name ;
    if (index > 0 ) {
      nameSuffix = name.substring(index + 1 , name.length()) ;
    }
    prefix_ = "[" + nameSuffix + "] "   ;
  }
  
  public String getLogCategory() { return category_ ; }
  
  public int  getLevel() { return level_  ; }
  public void setLevel(int level) { level_ = level ; }
  
  public void trace(Object message) { }

  public void trace(Object message, Throwable t) { }

  public void debug(Object message) {
    if (level_ >= DEBUG) {
      out_.print(prefix_) ; out_.print("[DEBUG] "); out_.println(message) ;
      addLogMessage(new LogMessage(prefix_ , DEBUG, message.toString(), null)) ;
    }
  }

  public void debug(Object message, Throwable t) {
    if (level_ >= DEBUG)  {
      out_.print(prefix_) ; out_.print("[DEBUG] "); out_.println(message) ;
      String strace = ExceptionUtil.getExoStackTrace(t) ;
      out_.println(strace) ;
      addLogMessage(new LogMessage(prefix_ , DEBUG, message.toString(), strace )) ;
    }
  }

  public void info(Object message) {
    if (level_ >= INFO) {
      out_.print(prefix_) ; out_.print("[INFO] "); out_.println(message) ;
      addLogMessage(new LogMessage(prefix_ , INFO, message.toString(), null)) ;
    }
  }

  public void info(Object message, Throwable t) {
    if (level_ >= INFO) {
      out_.print(prefix_) ; out_.print("[INFO] "); out_.println(message) ;
      String strace = ExceptionUtil.getExoStackTrace(t) ;
      out_.println(strace) ;
      addLogMessage(new LogMessage(prefix_ , DEBUG, message.toString(), strace )) ;
    }
  }

  public void warn(Object message) {
    if (level_ >= WARN) {
      out_.print(prefix_) ; out_.print("[WARN] "); out_.println(message) ;
      addLogMessage(new LogMessage(prefix_ , WARN, message.toString(), null )) ;
    }
  }

  public void warn(Object message, Throwable t) {
    if (level_ >= WARN) {
      out_.print(prefix_) ; out_.print("[WARN] "); out_.println(message) ;
      String strace = ExceptionUtil.getExoStackTrace(t) ;
      out_.println(strace) ;
      addLogMessage(new LogMessage(prefix_ , DEBUG, message.toString(), strace )) ;
    }
  }

  public void error(Object message) {
    if (level_ >= ERROR) {
      out_.print(prefix_) ; out_.print("[ERROR] "); out_.println(message) ;
      addLogMessage(new LogMessage(prefix_ , ERROR, message.toString(), null )) ;
    }
  }

  public void error(Object message, Throwable t) {
    if (level_ >= ERROR) {
      out_.print(prefix_) ; out_.print("[ERROR] "); out_.println(message) ;
      String strace = ExceptionUtil.getExoStackTrace(t) ;
      out_.println(strace) ;
      addLogMessage(new LogMessage(prefix_ , DEBUG, message.toString(), strace )) ;
    }
  }

  public void fatal(Object message) {
    if (level_ >= FATAL) {
      out_.print(prefix_) ; out_.print("[FATAL] "); out_.println(message) ;
      addLogMessage(new LogMessage(prefix_ , FATAL, message.toString(), null)) ;
    }
  }

  public void fatal(Object message, Throwable t) {
    if (level_ >= FATAL) {
      out_.print(prefix_) ; out_.print("[FATAL] "); out_.println(message) ;
      String strace = ExceptionUtil.getExoStackTrace(t) ;
      out_.println(strace) ;
      addLogMessage(new LogMessage(prefix_ , DEBUG, message.toString(), strace )) ;
    }
  }

  public final boolean isDebugEnabled() { return level_ >= DEBUG; }

  public final boolean isErrorEnabled() { return level_ >= ERROR; }

  public final boolean isFatalEnabled() { return level_ >= FATAL; }

  public final boolean isInfoEnabled() { return level_ >= INFO; }

  public final boolean isTraceEnabled() { return level_ >= TRACE; }

  public final boolean isWarnEnabled() { return level_ >= WARN; }
  
  private void addLogMessage(LogMessage lm) {
    logBuffer_.add(lm) ;
    if (logBuffer_.size() == LOG_BUFFER_SIZE * 2) {
      List list = new ArrayList(LOG_BUFFER_SIZE * 2) ;
      for (int i = LOG_BUFFER_SIZE; i < logBuffer_.size(); i++) {
        list.add(logBuffer_.get(i)) ;
      }
    }
    
    if (lm.getType() == ERROR) {
      errorBuffer_.add(lm) ;
      if (errorBuffer_.size() == ERROR_BUFFER_SIZE  * 2) {
        List list = new ArrayList(ERROR_BUFFER_SIZE * 2) ;
        for (int i = ERROR_BUFFER_SIZE; i < errorBuffer_.size(); i++) {
          list.add(errorBuffer_.get(i)) ;
        }
      }
    }
  }

  static public List getLogBuffer() { return logBuffer_ ; }
  static public List getErrorBuffer() { return errorBuffer_ ; }
}