/**
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.organization.ldap;


import java.util.*;

import net.sf.hibernate.Session;

import netscape.ldap.LDAPAttribute;
import netscape.ldap.LDAPAttributeSet;
import netscape.ldap.LDAPConnection;
import netscape.ldap.LDAPEntry;

import netscape.ldap.LDAPModification;
import netscape.ldap.LDAPModificationSet;
import netscape.ldap.LDAPSearchResults;
import org.apache.commons.lang.StringUtils;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.XResources;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.ldap.LDAPServiceContainer;
import org.exoplatform.services.organization.*;
import org.exoplatform.services.organization.impl.*;

/**
 * Created by the eXo platform team
 * User: Daniel Summer
 * Date: 28 august 2004
 *
 */
public class GroupLDAPHandler extends BaseLDAPHandler {
	public static final String BASEURL = OrganizationLDAPConfig.BASE_URL;
	public static final String PORTALURL = OrganizationLDAPConfig.PORTAL_URL;
	public static final String GROUPSURL = OrganizationLDAPConfig.GROUPS_URL;
	public static final String USERSURL = OrganizationLDAPConfig.USERS_URL;

	private LDAPServiceContainer serviceContainer_;
	private HibernateService hService_;
	private List listeners_;

	public GroupLDAPHandler(
		LDAPServiceContainer serviceContainer,
		HibernateService hService) {
		serviceContainer_ = serviceContainer;
		hService_ = hService;
		listeners_ = new ArrayList(5);
	}

	public void addGroupEventListener(GroupEventListener listener) {
		listeners_.add(listener);
	}

	public void createGroup(Group group) throws Exception {
		LDAPService service = null;
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String dn = "uid=" + group.getGroupName() + "," + GROUPSURL;
			GroupImpl g = (GroupImpl) group;
			String groupId = "/" + group.getGroupName();
			g.setId(groupId);
			LDAPAttributeSet attrs = new LDAPAttributeSet();

			String objectclass_values[] = { "top", "exogroup" };
			setutil("objectclass", objectclass_values, attrs, false);
			setutil("id", g.getId(), attrs);
			setutil("uid", g.getGroupName(), attrs);
			setutil("groupname", g.getGroupName(), attrs);

			LDAPEntry myEntry = new LDAPEntry(dn, attrs);
			preSave(g, true, session);
			service.add(myEntry);
			postSave(g, true, session);
			session.flush();

		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}
	}

	public void addChild(Group parent, Group child) throws Exception {
		LDAPService service = null;
		Session session = null;
		try {
			String MY_FILTER = "id=" + parent.getId();
			String MY_SEARCHBASE = GROUPSURL;

			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);

			List l = getUtil(res);
			LDAPEntry findEntry = (LDAPEntry) l.get(0);

			GroupImpl g = (GroupImpl) child;
			g.setParentId(getutil("id", findEntry.getAttributeSet()));

			if (findEntry != null) {
				String dn =
					"uid="
						+ g.getGroupName()
						+ ","
						+ "uid="
						+ getutil("groupname", findEntry.getAttributeSet())
						+ ","
						+ GROUPSURL;
				LDAPAttributeSet attrs = new LDAPAttributeSet();

				g.setId(g.getParentId() + "/" + child.getGroupName());
				String objectclass_values[] = { "top", "exogroup" };

				setutil("objectclass", objectclass_values, attrs, false);
				setutil("id", g.getId(), attrs);
				setutil("uid", g.getGroupName(), attrs);
				setutil("groupname", g.getGroupName(), attrs);

				LDAPEntry myEntry = new LDAPEntry(dn, attrs);
				preSave(g, true, session);
				service.add(myEntry);
				postSave(g, true, session);
				session.flush();
			} else {
				throw new Exception(
					"cannot find the parent group" + parent.getId());
			}
		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}
	}

	public void saveGroup(Group group) throws Exception {
		LDAPService service = null;
		Session session = null;
		GroupImpl groupImpl = (GroupImpl) group;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String olddn = groupImpl.getId();
			LDAPModificationSet mods = new LDAPModificationSet();
			mods.add(
				LDAPModification.REPLACE,
				new LDAPAttribute("groupname", groupImpl.getGroupName()));

			preSave(group, false, session);
			service.modify(olddn, mods);
			service.rename(olddn, "uid=" + groupImpl.getGroupName(), true);
			postSave(group, false, session);
			session.flush();
		} finally {
			hService_.closeSession(session);
			serviceContainer_.closeLDAPService(service);
		}
	}

	public Group removeGroup(Group group) throws Exception {
		LDAPService service = null;
		Session session = null;
		Group foundGroup = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String MY_FILTER = "id=" + group.getId();
			String MY_SEARCHBASE = GROUPSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);

			List l = getUtil(res);
			if (l.size() == 1) {
				LDAPEntry findEntry = (LDAPEntry) l.get(0);
				GroupImpl groupImpl = new GroupImpl();
				groupImpl.setId(getutil("id", findEntry.getAttributeSet()));
				groupImpl.setGroupName(
					getutil("groupname", findEntry.getAttributeSet()));
				groupImpl.setGroupName(
					getutil("description", findEntry.getAttributeSet()));
				preDelete(foundGroup, session);
				service.delete(findEntry.getDN());
				postDelete(foundGroup, session);
				session.flush();
			} else {
				throw new Exception(
					"Cannot find the group name " + group.getId());
			}

			//			delete cascade
			MY_FILTER = "membership=*";
			MY_SEARCHBASE = USERSURL;

			res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			while (res.hasMoreElements()) {
				LDAPEntry entry = res.next();

				Vector memberships =
					getutil("membership", entry.getAttributeSet(), false);
				for (Iterator i = memberships.iterator(); i.hasNext();) {
					String s = (String) i.next();
					String[] membership = StringUtils.split(s, ",");
					if (membership[3].equals(group.getGroupName())) {
						LDAPModificationSet mods = new LDAPModificationSet();
						mods.add(
							LDAPModification.DELETE,
							new LDAPAttribute("membership", s));
						service.modify(entry.getDN(), mods);
					}
				}

			}
		} finally {
			service = serviceContainer_.createLDAPService();
		}
		return foundGroup;
	}

	static void removeGroupEntry(
		String groupName,
		Session session,
		LDAPService service)
		throws Exception {
		String MY_FILTER = "uid=" + groupName;
		String MY_SEARCHBASE = GROUPSURL;
		String ENTRYDN = MY_FILTER + MY_SEARCHBASE;
		service.delete(ENTRYDN);
	}

	public Collection findGroupByMembership(
		String userName,
		String membershipType)
		throws Exception {
		LDAPService service = null;
		List groups = new ArrayList();
		List groupsImpl = new ArrayList();
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			String MY_SEARCHBASE = USERSURL;
			String MY_FILTER = "uid=" + userName;
			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);

			List l = getUtil(res);
			LDAPEntry findEntry = (LDAPEntry) l.get(0);

			LDAPAttributeSet attrs = findEntry.getAttributeSet();
			Vector memberships = getutil("membership", attrs, false);

			for (Iterator i = memberships.iterator(); i.hasNext();) {
				String s = (String) i.next();
				String[] membership = StringUtils.split(s, ",");
				if (membership[1].equals(membershipType)) {

					if (!groups.contains(membership[3])) {
						groups.add(membership[3]);
						GroupImpl groupImpl = new GroupImpl();
						//groupImpl.setId(findEntry.getDN());
						groupImpl.setGroupName(membership[3]);
						groupsImpl.add(groupImpl);
					}
				}
			}

		} finally {
			serviceContainer_.closeLDAPService(service);
		}
		return groupsImpl;
	}

	public Group findGroupByName(String groupName) throws Exception {
		Group group = null;
		LDAPService service = null;
		try {
			service = serviceContainer_.createLDAPService();
			List l = null;
			String MY_SEARCHBASE = GROUPSURL;
			String MY_FILTER = "groupname=" + groupName;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);

			l = getUtil(res);
			LDAPEntry findEntry = (LDAPEntry) l.get(0);

			LDAPAttributeSet attrs = findEntry.getAttributeSet();

			String groupname = getutil("groupname", attrs);
			GroupImpl groupImpl = new GroupImpl();
			groupImpl.setId(findEntry.getDN());
			groupImpl.setParentId(
				StringUtils.substringAfter(findEntry.getDN(), ","));
			groupImpl.setGroupName(groupname);

			return (Group) groupImpl;

		} finally {
			serviceContainer_.closeLDAPService(service);
		}

	}

	public Group findGroupById(String groupId) throws Exception {
		LDAPService service = null;
		try {
			service = serviceContainer_.createLDAPService();
			LDAPEntry findEntry = service.read(groupId);
			Group group = new GroupImpl();
			group.setGroupName(
				getutil("groupname", findEntry.getAttributeSet()));
			return group;
		} finally {
			serviceContainer_.closeLDAPService(service);
		}

	}

	public Collection findGroups(Group parent) throws Exception {
		Collection groups = null;
		LDAPService service = null;
		try {
			service = serviceContainer_.createLDAPService();
			String MY_SEARCHBASE = GROUPSURL;
			if (parent == null) {

				LDAPSearchResults res =
					service.search(
						MY_SEARCHBASE,
						LDAPConnection.SCOPE_SUB,
						null,
						null,
						false);
				List l = getUtil(res);

				for (Iterator i = l.iterator(); i.hasNext();) {
					LDAPEntry entry = (LDAPEntry) i.next();
					GroupImpl g = new GroupImpl();
					g.setGroupName(
						getutil("groupname", entry.getAttributeSet()));
					groups.add(g);
				}

			} else {
				String MY_SEARCHBASE2 = "groupname=" + parent + MY_SEARCHBASE;
				String MY_FILTER = "groupname=" + parent;
				LDAPSearchResults res =
					service.search(
						MY_SEARCHBASE2,
						LDAPConnection.SCOPE_SUB,
						null,
						null,
						false);
				List l = getUtil(res);

				if (l.size() == 1) {
					for (Iterator i = l.iterator(); i.hasNext();) {
						LDAPEntry entry = (LDAPEntry) i.next();
						GroupImpl g = new GroupImpl();
						g.setGroupName(
							getutil("groupname", entry.getAttributeSet()));
						groups.add(g);
					}
				} else {
					throw new Exception(
						"Cannot find the group parent " + parent);
				}
			}
		} finally {
			serviceContainer_.closeLDAPService(service);
		}
		return groups;
	}

	public Collection findGroupsOfUser(String user) throws Exception {
		List groups = new ArrayList();
		LDAPService service = null;
		try {
			service = serviceContainer_.createLDAPService();
			String MY_SEARCHBASE = USERSURL;
			String MY_FILTER = "uid=" + user;
			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);

			List l = getUtil(res);
			LDAPEntry findEntry = (LDAPEntry) l.get(0);

			LDAPAttributeSet attrs = findEntry.getAttributeSet();
			Vector memberships = getutil("membership", attrs, false);

			for (Iterator i = memberships.iterator(); i.hasNext();) {
				String s = (String) i.next();
				String[] membership = StringUtils.split(s, ",");
				if (!groups.contains(membership[3])) {
					groups.add(membership[3]);
				}
			}

		} finally {
			serviceContainer_.closeLDAPService(service);
		}
		return groups;
	}

	private void preSave(Group group, boolean isNew, Session session)
		throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			GroupEventListener listener =
				(GroupEventListener) listeners_.get(i);
			listener.preSave(group, isNew, xresources);
		}
	}

	private void postSave(Group group, boolean isNew, Session session)
		throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			GroupEventListener listener =
				(GroupEventListener) listeners_.get(i);
			listener.postSave(group, isNew, xresources);
		}
	}

	private void preDelete(Group group, Session session) throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			GroupEventListener listener =
				(GroupEventListener) listeners_.get(i);
			listener.preDelete(group, xresources);
		}
	}

	private void postDelete(Group group, Session session) throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			GroupEventListener listener =
				(GroupEventListener) listeners_.get(i);
			listener.postDelete(group, xresources);
		}
	}

}