/**
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.organization.ldap;



import java.util.*;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.ldap.LDAPServiceContainer;
import org.exoplatform.services.organization.*;
import org.exoplatform.services.organization.impl.MembershipTypeImpl;

import net.sf.hibernate.Session;
import netscape.ldap.LDAPAttribute;
import netscape.ldap.LDAPAttributeSet;
import netscape.ldap.LDAPConnection;
import netscape.ldap.LDAPEntry;
import netscape.ldap.LDAPModification;
import netscape.ldap.LDAPModificationSet;
import netscape.ldap.LDAPSearchResults;

/**
 * Created by the eXo platform team
 * User: Daniel Summer
 * Date: 28 august 2004
 *
 */
public class MembershipTypeLDAPHandler extends BaseLDAPHandler {
	private final String BASEURL = OrganizationLDAPConfig.BASE_URL;
	private final String PORTALURL = OrganizationLDAPConfig.PORTAL_URL;
	private final String ROLESURL = OrganizationLDAPConfig.ROLES_URL;
	private final String USERSURL = OrganizationLDAPConfig.USERS_URL;
	private final String MEMBERSHIPSURL =
		OrganizationLDAPConfig.MEMBERSHIPS_URL;
	private LDAPServiceContainer serviceContainer_;
	private HibernateService hService_;

	public MembershipTypeLDAPHandler(
		LDAPServiceContainer serviceContainer,
		HibernateService hService) {
		serviceContainer_ = serviceContainer;
		hService_ = hService;
	}

	public MembershipType createMembershipType(MembershipType mt)
		throws Exception {
		LDAPService service = null;
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			Date now = new Date();
			mt.setCreatedDate(now);
			mt.setModifiedDate(now);
			String ENTRYDN = "uid=" + mt.getName() + "," + MEMBERSHIPSURL;

			LDAPAttributeSet attrs = new LDAPAttributeSet();
			String objectclass_values[] = { "exomembershiptype" };
			setutil("objectclass", objectclass_values, attrs, false);
			setutil("uid", mt.getName(), attrs);
			setutil("descriptions", mt.getDescription(), attrs);
			setutil("createddate", mt.getCreatedDate().toGMTString(), attrs);
			setutil("modifieddate", mt.getModifiedDate().toGMTString(), attrs);

			LDAPEntry myEntry = new LDAPEntry(ENTRYDN, attrs);
			service.add(myEntry);
			session.flush();
		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}
		return mt;
	}

	public MembershipType saveMembershipType(MembershipType mt)
		throws Exception {
		LDAPService service = null;

		try {
			service = serviceContainer_.createLDAPService();

			Date now = new Date();
			mt.setModifiedDate(now);
			String ENTRYDN = "uid=" + mt.getName() + "," + MEMBERSHIPSURL;

			LDAPModificationSet mods = new LDAPModificationSet();
			mods.add(
				LDAPModification.REPLACE,
				new LDAPAttribute("descriptions", mt.getDescription()));
			mods.add(
				LDAPModification.REPLACE,
				new LDAPAttribute("modifieddate", now.toGMTString()));
			service.modify(ENTRYDN, mods);

		} finally {
			serviceContainer_.closeLDAPService(service);
		}
		return mt;
	}

	public MembershipType findMembershipType(String name) throws Exception {
		LDAPService service = null;
		try {
			service = serviceContainer_.createLDAPService();
			List l = null;
			String ENTRYDN = "uid=" + name + "," + MEMBERSHIPSURL;
			String MY_FILTER = "uid=" + name;
			String MY_SEARCHBASE = MEMBERSHIPSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			if (!res.hasMoreElements()) {
				return null;
			}
			LDAPEntry findEntry = service.read(ENTRYDN);

			LDAPAttributeSet attrs = findEntry.getAttributeSet();

			MembershipTypeImpl membershipTypeImpl = new MembershipTypeImpl();
			membershipTypeImpl.setName(getutil("uid", attrs));
			membershipTypeImpl.setDescription(getutil("descriptions", attrs));
			membershipTypeImpl.setCreatedDate(
				new Date(getutil("createddate", attrs)));
			membershipTypeImpl.setModifiedDate(
				new Date(getutil("modifieddate", attrs)));
			return membershipTypeImpl;
		} finally {
			serviceContainer_.closeLDAPService(service);
		}
	}

	public MembershipType removeMembershipType(String name) throws Exception {
		LDAPService service = null;
		MembershipType m = null;
		try {
			service = serviceContainer_.createLDAPService();
			String ENTRYDN = "uid=" + name + "," + MEMBERSHIPSURL;
			LDAPEntry findEntry = service.read(ENTRYDN);
			LDAPAttributeSet attrs = findEntry.getAttributeSet();
			m = new MembershipTypeImpl();
			m.setName(getutil("uid", attrs));
			m.setDescription(getutil("description", attrs));
			m.setCreatedDate(new Date(getutil("createddate", attrs)));
			m.setModifiedDate(new Date(getutil("modifieddate", attrs)));
			service.delete(ENTRYDN);

			//delete cascade
			String MY_FILTER = "membership=*";
			String MY_SEARCHBASE = USERSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			while (res.hasMoreElements()) {
				LDAPEntry entry = res.next();
				
				Vector memberships =
					getutil("membership", entry.getAttributeSet(), false);
				for (Iterator i = memberships.iterator(); i.hasNext();) {
					String s = (String) i.next();
					String[] membership = StringUtils.split(s, ",");
					if (membership[1].equals(name)) {
						LDAPModificationSet mods = new LDAPModificationSet();
						mods.add(
							LDAPModification.DELETE,
							new LDAPAttribute("membership", s));
						service.modify(entry.getDN(), mods);
					}
				}

			}

		} finally {
			serviceContainer_.closeLDAPService(service);
		}
		return m;
	}

	static void removeMembershipTypeEntry(
		String name,
		Session session,
		LDAPService service)
		throws Exception {

		String ENTRYDN =
			"uid=" + name + "," + OrganizationLDAPConfig.MEMBERSHIPS_URL;
		service.delete(ENTRYDN);

	}

	public Collection findMembershipTypes() throws Exception {
		Collection memberships = null;
		LDAPService service = null;

		try {
			service = serviceContainer_.createLDAPService();
			String MY_SEARCHBASE = MEMBERSHIPSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					null,
					null,
					false);
			while (res.hasMoreElements()) {
				LDAPEntry findEntry = null;
				findEntry = res.next();
				LDAPAttributeSet attrs = findEntry.getAttributeSet();
				MembershipTypeImpl m = new MembershipTypeImpl();
				m.setName(getutil("uid", attrs));
				m.setDescription(getutil("descriptions", attrs));
				m.setCreatedDate(new Date(getutil("createddate", attrs)));
				m.setModifiedDate(new Date(getutil("modifieddate", attrs)));
				memberships.add(m);
			}
		} finally {
			serviceContainer_.closeLDAPService(service);
		}

		return memberships;
	}
}