/**
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.organization.ldap;



import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.services.organization.*;
import org.exoplatform.services.organization.impl.*;
import org.picocontainer.Startable;

/**
 * Created y the eXo platform team
 * User: Daniel Summer
 * Date: 28 august 2004
 *
 */
public class OrganizationServiceLDAPImpl
	implements OrganizationService, Startable {

	//private LDAPServiceContainer ldapServiceContainer_;

	UserLDAPHandler userLDAPHandler_;
	UserProfileLDAPHandler userProfileLDAPHandler_;
	GroupLDAPHandler groupLDAPHandler_;
	MembershipLDAPHandler membershipLDAPHandler_;
	MembershipTypeLDAPHandler membershipTypeLDAPHandler_;

	private List listeners_;

	public OrganizationServiceLDAPImpl(
		UserLDAPHandler userLDAPHandler,
		GroupLDAPHandler groupLDAPHandler,
		MembershipLDAPHandler membershipLDAPHandler,
		MembershipTypeLDAPHandler membershipTypeLDAPHandler,
		UserProfileLDAPHandler userProfileLDAPHandler) {

		userLDAPHandler_ = userLDAPHandler;
		userProfileLDAPHandler_ = userProfileLDAPHandler;
		groupLDAPHandler_ = groupLDAPHandler;
		membershipLDAPHandler_ = membershipLDAPHandler;
		membershipTypeLDAPHandler_ = membershipTypeLDAPHandler;
		listeners_ = new ArrayList();
		//ldapServiceContainer_ = ldapServiceContainer;
	}

	public void start() {
		for (int i = 0; i < listeners_.size(); i++) {
			OrganizationServiceListener listener =
				(OrganizationServiceListener) listeners_.get(i);
			listener.inititalize(this);
		}
	}

	public void stop() {
	}

	synchronized public void addListener(OrganizationServiceListener listener) {
		listeners_.add(listener);
	}

	synchronized public void removeListener(OrganizationServiceListener listener) {
		listeners_.remove(listener);
	}

	public User createUserInstance() {
		return new UserImpl();
	}

	final public List getUserEventListeners() {
		return userLDAPHandler_.getUserEventListeners();
	}

	public void addUserEventListener(UserEventListener listener) {
		userLDAPHandler_.addUserEventListener(listener);
	}

	final public void createUser(User user) throws Exception {
		userLDAPHandler_.createUser(user);
	}

	final public void saveUser(User user) throws Exception {
		userLDAPHandler_.saveUser(user);
	}

	final public User removeUser(String user) throws Exception {
		return userLDAPHandler_.removeUser(user);
	}

	final public User findUserByName(String userName) throws Exception {
		return userLDAPHandler_.findUserByName(userName);
	}

	public PageList getUserPageList(int pageSize) throws Exception {
		return userLDAPHandler_.getUserPageList(pageSize);
	}

	final public PageList findUsers(Query query) throws Exception {
		return userLDAPHandler_.findUsers(query);
	}

	final public PageList findUsersByGroup(String groupId) throws Exception {
		return userLDAPHandler_.findUsersByGroup(groupId);
	}

	public void addUserProfileEventListener(UserProfileEventListener listener) {
		userProfileLDAPHandler_.addUserProfileEventListener(listener);
	}

	final public UserProfile createUserProfileInstance() {
		return new UserProfileImpl();
	}

	final public void saveUserProfile(UserProfile profile) throws Exception {
		userProfileLDAPHandler_.saveUserProfile(profile);
	}

	final public UserProfile removeUserProfile(String user) throws Exception {
		return userProfileLDAPHandler_.removeUserProfile(user);
	}

	final public UserProfile findUserProfileByName(String userName)
		throws Exception {
		return userProfileLDAPHandler_.findUserProfileByName(userName);
	}

	final public Collection findUserProfiles() throws Exception {
		return userProfileLDAPHandler_.findUserProfiles();
	}

	public void addGroupEventListener(GroupEventListener listener) {
		groupLDAPHandler_.addGroupEventListener(listener);
	}

	final public Group createGroupInstance() {
		return new GroupImpl();
	}

	final public void createGroup(Group group) throws Exception {
		groupLDAPHandler_.createGroup(group);
	}

	final public void addChild(Group parent, Group child) throws Exception {
		groupLDAPHandler_.addChild(parent, child);
	}

	final public void saveGroup(Group group) throws Exception {
		groupLDAPHandler_.saveGroup(group);
	}

	final public Group removeGroup(Group group) throws Exception {
		return groupLDAPHandler_.removeGroup(group);
	}

	final public Collection findGroupByMembership(
		String userName,
		String membershipType)
		throws Exception {
		return groupLDAPHandler_.findGroupByMembership(
			userName,
			membershipType);
	}

	final public Group findGroupByName(String groupName) throws Exception {
		return groupLDAPHandler_.findGroupByName(groupName);
	}

	public Group findGroupById(String groupId) throws Exception {
		return groupLDAPHandler_.findGroupById(groupId);
	}

	final public Collection findGroups(Group parent) throws Exception {
		return groupLDAPHandler_.findGroups(parent);
	}

	final public Collection findGroupsOfUser(String user) throws Exception {
		return groupLDAPHandler_.findGroupsOfUser(user);
	}

	public void addMembershipEventListener(MembershipEventListener listener) {
		membershipLDAPHandler_.addMembershipEventListener(listener);
	}

	final public Membership createMembershipInstance() {
		return new MembershipImpl();
	}

	final public void createMembership(Membership m) throws Exception {
		membershipLDAPHandler_.createMembership(m);
	}

	final public void linkMembership(
		String userName,
		Group group,
		Membership m)
		throws Exception {
		membershipLDAPHandler_.linkMembership(userName, group, m);
	}

	final public void saveMembership(Membership m) throws Exception {
		membershipLDAPHandler_.saveMembership(m);
	}

	final public Membership removeMembership(String id) throws Exception {
		return membershipLDAPHandler_.removeMembership(id);
	}

	public Membership findMembershipByUserGroupAndType(
		String userName,
		String groupId,
		String type)
		throws Exception {
		return membershipLDAPHandler_.findMembershipByUserGroupAndType(
			userName,
			groupId,
			type);
	}

	final public Collection findMembershipsByUserAndGroup(
		String userName,
		String groupId)
		throws Exception {
		return membershipLDAPHandler_.findMembershipsByUserAndGroup(
			userName,
			groupId);
	}

	final public Collection findMembershipsByUser(String userName)
		throws Exception {
		return membershipLDAPHandler_.findMembershipsByUser(userName);
	}

	final public Collection findMembershipsByGroup(Group group)
		throws Exception {
		return membershipLDAPHandler_.findMembershipsByGroup(group);
	}

	final public Membership findMembership(String id) throws Exception {
		return membershipLDAPHandler_.findMembership(id);
	}

	final public MembershipType createMembershipTypeInstance() {
		return new MembershipTypeImpl();
	}

	final public MembershipType createMembershipType(MembershipType mt)
		throws Exception {
		return membershipTypeLDAPHandler_.createMembershipType(mt);
	}

	final public MembershipType saveMembershipType(MembershipType mt)
		throws Exception {
		return membershipTypeLDAPHandler_.saveMembershipType(mt);
	}

	final public MembershipType findMembershipType(String name)
		throws Exception {
		return membershipTypeLDAPHandler_.findMembershipType(name);
	}

	final public MembershipType removeMembershipType(String name)
		throws Exception {
		return membershipTypeLDAPHandler_.removeMembershipType(name);
	}

	final public Collection findMembershipTypes() throws Exception {
		return membershipTypeLDAPHandler_.findMembershipTypes();
	}
}