/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.impl;

import org.apache.commons.logging.Log;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.portal.PortalInfo;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.portal.model.*;
import org.exoplatform.services.portletcontainer.pci.ExoWindowID;
import org.exoplatform.services.portletcontainer.pci.WindowID;
import org.exoplatform.services.portletcontainer.pci.model.ExoPortletPreferences;
import org.exoplatform.services.portletcontainer.persistence.PortletPreferencesPersister;

/**
 * Jul 16, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: PortletPreferencesPersisterImpl.java,v 1.4 2004/08/11 02:22:16 tuan08 Exp $
 */
public class PortletPreferencesPersisterImpl implements  PortletPreferencesPersister{
  private PortalConfigServiceImpl service_ ;
  private Log log_;

	public PortletPreferencesPersisterImpl(PortalConfigServiceImpl impl,
                                         LogService lservice) {
		service_ = impl ;
    log_ = lservice.getLog(getClass()); 
  }

	public ExoPortletPreferences getPortletPreferences(WindowID windowID) throws Exception {
		ExoWindowID exoWindowID = (ExoWindowID) windowID ;
		String pageRefId = exoWindowID.getPageReferenceId() ;
		if(pageRefId != null) {
			Page page = service_.getPage(pageRefId) ;
			Portlet portlet = page.findPortletByWindowId(exoWindowID.getPersistenceId()) ;
			if(portlet != null) {
				return portlet.getPortletPreferences() ;
			}
			return null ;
		}

		PortalConfig config = service_.getPortalConfig(windowID.getOwner()) ;
		PortalInfo pinfo = SessionContainer.getInstance().getPortalInfo() ;
		Container layout = null ;
		if(PortalInfo.XHTMLMP_MIME_TYPE.equals(pinfo.getPreferredMimeType())) {
		  layout = config.getMobilePortalLayout() ;
		} else {
		  layout = config.getLayout() ;
		}
		Portlet portlet = layout.findPortletByWindowId(exoWindowID.getPersistenceId()) ;
		if(portlet != null) {
			return portlet.getPortletPreferences() ;
		}
		return null  ;
	}

	public void savePortletPreferences(WindowID windowID, 
																		 ExoPortletPreferences exoPref) throws Exception {
		ExoWindowID exoWindowID = (ExoWindowID) windowID ;
		String pageRefId = exoWindowID.getPageReferenceId() ;
		if(pageRefId != null) {
			Page page = service_.getPage(pageRefId) ;
			Portlet portlet = page.findPortletByWindowId(exoWindowID.getPersistenceId()) ;
			if(portlet == null) {
				log_.error("cannot find the portlet, portlet persitence id: " + exoWindowID.getPersistenceId()) ;
				throw new Exception("Cannot find the portlet in page " + pageRefId) ;
			}
			portlet.setPortletPreferences(exoPref) ;
			service_.savePage(page) ;
		} else {
			PortalConfig config = service_.getPortalConfig(windowID.getOwner()) ;
			PortalInfo pinfo = SessionContainer.getInstance().getPortalInfo() ;
			Container layout = null ;
			if(PortalInfo.XHTMLMP_MIME_TYPE.equals(pinfo.getPreferredMimeType())) {
			  layout = config.getMobilePortalLayout() ;
			} else {
			  layout = config.getLayout() ;
			}
			Portlet portlet = layout.findPortletByWindowId(exoWindowID.getPersistenceId()) ;
			if(portlet == null) {
				log_.error("cannot find the portlet, portlet persitence id: " + exoWindowID.getPersistenceId()) ;
				throw new Exception("Cannot find the portlet in layout config") ;
			}
			portlet.setPortletPreferences(exoPref) ;
			service_.savePortalConfig(config) ;
		}
	}
}