/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.log.impl;

import java.util.LinkedList;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.monitor.ActionData;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.ObjectQuery;
import org.exoplatform.services.idgenerator.IDGeneratorService;
import org.exoplatform.services.portal.log.PortalLogService;
import org.exoplatform.services.portal.log.Query;
import org.exoplatform.services.portal.log.SessionLogData;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.XppDriver;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Dec 2, 2004
 * @version $Id$
 */
public class PortalLogServiceImpl implements PortalLogService {
  static private XStream xstream_ ;
  static private String[] MAPPING =
  { "org/exoplatform/services/portal/log/impl/SessionLogDataDescriptionImpl.hbm.xml",
    "org/exoplatform/services/portal/log/impl/SessionLogDataImpl.hbm.xml" } ;
  
  private HibernateService hservice_ ;
  private IDGeneratorService idService_ ;
  
  public PortalLogServiceImpl(HibernateService hservice, IDGeneratorService idService) {
    hservice.addMappingFiles(MAPPING) ;
    hservice_ = hservice ;
    idService_ = idService ;
  }
  
  public SessionLogData getSessionLogData(String id) throws Exception {
    return (SessionLogData)hservice_.findOne(SessionLogDataImpl.class, id) ;
  }

  public void saveSessionLogData(SessionLogData data) throws Exception {
    SessionLogDataImpl impl = (SessionLogDataImpl) data ;
    if(impl.getId() == null) impl.setId(idService_.generateStringID(impl)) ;
    hservice_.create(impl) ;
  }

  public PageList getSessionLogDatas(Query query) throws Exception {
    ObjectQuery oq = new ObjectQuery(SessionLogDataDescriptionImpl.class);
    String remoteUser = query.getRemoteUser() ;
    if(remoteUser != null && remoteUser.length() > 0) {
      remoteUser = remoteUser.replace('*', '%');
      oq.addLIKE("remoteUser", remoteUser) ;
    }
    String ipAddress = query.getIPAddress() ;
    if(ipAddress != null && ipAddress.length() > 0)  {
      ipAddress = ipAddress.replace('*', '%') ;
      oq.addLIKE("IPAddress", ipAddress) ;
    }
    if(query.getFromDate() != null) {
      oq.addGT("accessTime", Long.toString(query.getFromDate().getTime())) ;
    }
    if(query.getToDate() != null) {
      oq.addLT("accessTime", Long.toString(query.getToDate().getTime())) ;
    }
    if(query.getFromDuration() > 0) {
      oq.addGT("duration", Long.toString(query.getFromDuration())) ;
    }
    if(query.getFromDuration() > 0) {
      oq.addLT("duration", Long.toString(query.getToDuration())) ;
    }
    return new DBObjectPageList(hservice_, oq) ;
  }
  
  static public XStream getXStreamInstance() {
    if (xstream_ == null ) {
      xstream_ = new XStream(new XppDriver());
      xstream_.alias("action-data", ActionData.class);
      xstream_.alias("session-data", LinkedList.class);
    }
    return xstream_ ;
  }
}
