/**
 **************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.xml.transform.impl.html;

//import org.exoplatform.services.xml.transform.TransformRules;
import org.exoplatform.services.xml.transform.html.HtmlTransformer;
import org.w3c.tidy.Tidy;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.URIResolver;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamResult;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

/**
 * Created by The eXo Platform SARL        .
 * 
 * Tidying incoming HTML to XHTML result
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TidyTransformer.java,v 1.5 2004/03/15 09:36:32 geaz Exp $
 */
public class TidyTransformer implements HtmlTransformer {
    protected Tidy tidy;
    protected Properties props;

    public TidyTransformer() {
        tidy = new Tidy();
        initProps();
    }

//    public void initTransformRules(TransformRules rules) {}

    public void transform(Source source, Result result) throws TransformerException {
        //        if(!(source instanceof StreamSource))
        //            throw new TransformerException("TidyTransformer.transform() failed! Only StreamSource type is supported!");
        InputStream is = SAXSource.sourceToInputSource(source).getByteStream();
        if (result instanceof StreamResult) {
            OutputStream os = ((StreamResult)result).getOutputStream();
            if (os == null)
                throw new TransformerException("TidyTransformer.transform() failed! Temporary only OutputStream builded Result is supported!");
            else
                tidy.parse(is, os);
        } else
            throw new TransformerException("TidyTransformer.transform() failed! Only StreamResult type is supported!");
    }

    public Object getParameter(String param) {
        return getOutputProperty(param);
    }

  /**
   * Sets property for Tidy parser (the same as setOutputProperty() )
   * See Tidy properties
   */
    public void setParameter(String name, Object value) {
        setOutputProperty(name, (String) value);
    }

    public void clearParameters() {
        initProps();
    }

    public String getOutputProperty(String prop) {
        return props.getProperty(prop);
    }

  /**
   * Sets property for Tidy parser
   * See Tidy properties
   */
    public void setOutputProperty(String name, String value) {
        props.setProperty(name, value);
        tidy.setConfigurationFromProps(props);
    }

  /**
   * Sets properties for Tidy parser
   * See Tidy properties
   */
    public void setOutputProperties(Properties props) {
        this.props = props;
        tidy.setConfigurationFromProps(props);
    }

    public Properties getOutputProperties() {
        return props;
    }

    public URIResolver getURIResolver() {
        // make no sence here
        return null;
    }

    public void setURIResolver(URIResolver resolver) {
        // make no sence here
    }

    // for debugging only
//    public Tidy getTidyParser() {
//        return tidy;
//    }

    /** @link dependency */

    /*# Tidy lnkTidy; */

    private void initProps() {

        this.props = new Properties();

        props.setProperty("quiet", "true");
        props.setProperty("quote-ampersand", "true");
        props.setProperty("output-xhtml", "true");
        props.setProperty("show-warnings", "false");
        props.setProperty("clean", "true");

        tidy.setConfigurationFromProps(props);

    }
}
