/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.xml.transform.impl.pipe;

import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Properties;
import java.util.HashMap;
import java.util.Iterator;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.URIResolver;
import javax.xml.transform.Source;
import javax.xml.transform.Result;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.TransformerException;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.parsers.SAXParserFactory;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.xml.resolving.SimpleResolvingService;
import org.exoplatform.services.xml.resolving.XMLResolvingService;
import org.exoplatform.services.xml.resolving.impl.simple.SimpleDirResolvingServiceImpl;
import org.exoplatform.services.xml.transform.Transformer;
import org.exoplatform.services.xml.transform.pipe.EmptyPipeException;
import org.exoplatform.services.xml.transform.pipe.PipeTransformer;
import org.xml.sax.InputSource;
import org.xml.sax.EntityResolver;
import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;


import java.util.ArrayList;


/**
 * Created by The eXo Platform SARL        .
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: PipeTransformerImpl.java,v 1.9 2004/07/24 16:39:15 tuan08 Exp $
 */
public class PipeTransformerImpl implements PipeTransformer {

  private ArrayList rules;
  private XMLReader reader;
  private HashMap params;
  private Properties outProps;
  private URIResolver uriResolver;

  public PipeTransformerImpl() throws SAXException {

    rules = new ArrayList();
    this.params = new HashMap();
    try {
      SAXParserFactory parserFactory = SAXParserFactory.newInstance();
      parserFactory.setNamespaceAware(true);
      this.reader = parserFactory.newSAXParser().getXMLReader();
/*
      PortalContainer manager = PortalContainer.getInstance();
      XMLResolvingService service = (XMLResolvingService) manager.
          getComponentInstanceOfType(SimpleResolvingService.class);
      this.reader.setEntityResolver(service.getEntityResolver());
*/
    } catch (Exception e) {
      throw new SAXException("PipeTransformerImpl() Failed. Reason : " + e);
    }

  }

  public PipeTransformer addTransformRule(Transformer transformer) {
    rules.add(transformer);
    return this;
  }

  public PipeTransformer insertTransformRule(int index, Transformer transformer) {
    rules.add(index, transformer);
    return this;
  }

  public void transform(Source source, Result result) throws TransformerException {
    byte[] buffer;
//        if (rules == null)
//            throw new TransformerException("PipeTransformer.transform() failed! Rules not initialized! Use initTransformRules(RULES) first.");
    try {
      // TODO DOM & SAX
      if (source instanceof StreamSource) {
        InputStream is = ((StreamSource) source).getInputStream();
        if (is == null)
          throw new TransformerException("PipeTransformer.transform() failed! Temporary only InputStream builded StreamSource is supported!");
        else {
          buffer = new byte[is.available()];
//          System.out.println("is Available  : " + is.available());
          is.read(buffer);
//          System.out.println("content entrance : " + new String(buffer));
        }
      } else {
        throw new TransformerException("PipeTransformerImpl.transform() failed! Temporary only StreamSource is supported. But have " + source);
      }
//            Iterator iter = rules.iterator();
//            while (rules.hasNextTransformer()) {
      for (int i = 0; i < rules.size(); i++) {
//        System.out.println("Rule Index : " + i);
        Transformer transformer = (Transformer) rules.get(i);
        InputSource is = new InputSource(new ByteArrayInputStream(buffer));
        SAXSource src = new SAXSource(reader, is);
        initStuffs(transformer);
        if (i == (rules.size() - 1)) {
          if (this.outProps != null)
            transformer.setOutputProperties(outProps);
          transformer.transform(src, result);
        } else {
          StreamResult tmpResult = new StreamResult(new ByteArrayOutputStream());
          transformer.transform(src, tmpResult);
          buffer = ((ByteArrayOutputStream) tmpResult.getOutputStream()).toByteArray();
//          System.out.println("Content : " + new String(((ByteArrayOutputStream) tmpResult.getOutputStream()).toByteArray()));
//          System.out.println("value : " + new String(buffer));
//          System.out.println("Length : " + buffer.length);

        }
      }
    } catch (Exception e) {
      throw new TransformerException("PipeTransformerImpl.transform() failed!", e);
    }
  }

  public Object getParameter(String param) {
    return this.params.get(param);
  }

  public void setParameter(String name, Object value) {
    this.params.put(name, value);
  }

  public void clearParameters() {
    this.params = new HashMap();
  }

  public String getOutputProperty(String prop) {
    return this.outProps.getProperty(prop);
  }

  public void setOutputProperty(String name, String value) {
    this.outProps.setProperty(name, value);
  }

  public void setOutputProperties(Properties props) {
    this.outProps = props;
  }

  public Properties getOutputProperties() {
    return this.outProps;
  }

  public URIResolver getURIResolver() {
    return this.uriResolver;
  }

  public void setURIResolver(URIResolver resolver) {
    this.uriResolver = resolver;
  }

  private void initStuffs(Transformer transformer) {
    if (this.uriResolver != null)
      transformer.setURIResolver(this.uriResolver);
    Iterator keys = params.keySet().iterator();
    while (keys.hasNext()) {
      String key = (String) keys.next();
      transformer.setParameter(key, params.get(key));
    }

  }

}
